import io
from pathlib import Path
from warnings import warn

from dynaconf import default_settings
from dynaconf.constants import YAML_EXTENSIONS
from dynaconf.loaders.base import BaseLoader
from dynaconf.utils import object_merge
from dynaconf.utils.parse_conf import try_to_encode
from dynaconf.vendor.ruamel import yaml

# Add support for Dynaconf Lazy values to YAML dumper
yaml.SafeDumper.yaml_representers[
    None
] = lambda self, data: yaml.representer.SafeRepresenter.represent_str(
    self, try_to_encode(data)
)


def load(obj, env=None, silent=True, key=None, filename=None):
    """
    Reads and loads in to "obj" a single key or all keys from source file.

    :param obj: the settings instance
    :param env: settings current env default='development'
    :param silent: if errors should raise
    :param key: if defined load a single key, else load all in env
    :param filename: Optional custom filename to load
    :return: None
    """
    # Resolve the loaders
    # https://github.com/yaml/pyyaml/wiki/PyYAML-yaml.load(input)-Deprecation
    # Possible values are `safe_load, full_load, unsafe_load, load`
    yaml_reader = getattr(
        yaml, obj.get("YAML_LOADER_FOR_DYNACONF"), yaml.safe_load
    )
    if yaml_reader.__name__ == "unsafe_load":  # pragma: no cover
        warn(
            "yaml.unsafe_load is deprecated."
            " Please read https://msg.pyyaml.org/load for full details."
            " Try to use full_load or safe_load."
        )

    loader = BaseLoader(
        obj=obj,
        env=env,
        identifier="yaml",
        extensions=YAML_EXTENSIONS,
        file_reader=yaml_reader,
        string_reader=yaml_reader,
    )
    loader.load(filename=filename, key=key, silent=silent)


def write(settings_path, settings_data, merge=True):
    """Write data to a settings file.

    :param settings_path: the filepath
    :param settings_data: a dictionary with data
    :param merge: boolean if existing file should be merged with new data
    """
    settings_path = Path(settings_path)
    if settings_path.exists() and merge:  # pragma: no cover
        with io.open(
            str(settings_path), encoding=default_settings.ENCODING_FOR_DYNACONF
        ) as open_file:
            object_merge(yaml.safe_load(open_file), settings_data)

    with io.open(
        str(settings_path),
        "w",
        encoding=default_settings.ENCODING_FOR_DYNACONF,
    ) as open_file:
        yaml.dump(
            settings_data,
            open_file,
            Dumper=yaml.dumper.SafeDumper,
            explicit_start=True,
            indent=2,
            default_flow_style=False,
        )



from collections import OrderedDict
from .  import TomlEncoder
from .  import TomlDecoder
class TomlOrderedDecoder(TomlDecoder):
	def __init__(A):super(A.__class__,A).__init__(_dict=OrderedDict)
class TomlOrderedEncoder(TomlEncoder):
	def __init__(A):super(A.__class__,A).__init__(_dict=OrderedDict)


from __future__ import print_function,absolute_import,division,unicode_literals
_o='\u2028\u2029'
_n='\r\n'
_m='\r\n\x85'
_l='while scanning a quoted scalar'
_k='0123456789ABCDEFabcdef'
_j=' \r\n\x85\u2028\u2029'
_i='\x07'
_h='expected a comment or a line break, but found %r'
_g='directive'
_f='\ufeff'
_e="could not find expected ':'"
_d='while scanning a simple key'
_c='typ'
_b='\\'
_a='\t'
_Z="expected ' ', but found %r"
_Y='while scanning a %s'
_X='\r\n\x85\u2028\u2029'
_W='while scanning a block scalar'
_V='expected alphabetic or numeric character, but found %r'
_U='a'
_T='...'
_S='---'
_R='>'
_Q='9'
_P='"'
_O=':'
_N='-'
_M=' \t'
_L='\x00 \r\n\x85\u2028\u2029'
_K='0'
_J="'"
_I='\x00'
_H='!'
_G='while scanning a directive'
_F='#'
_E='\n'
_D=' '
_C=None
_B=False
_A=True
from .error import MarkedYAMLError
from .tokens import *
from .compat import utf8,unichr,PY3,check_anchorname_char,nprint
if _B:from typing import Any,Dict,Optional,List,Union,Text;from .compat import VersionType
__all__=['Scanner','RoundTripScanner','ScannerError']
_THE_END='\n\x00\r\x85\u2028\u2029'
_THE_END_SPACE_TAB=' \n\x00\t\r\x85\u2028\u2029'
_SPACE_TAB=_M
class ScannerError(MarkedYAMLError):0
class SimpleKey:
	def __init__(self,token_number,required,index,line,column,mark):self.token_number=token_number;self.required=required;self.index=index;self.line=line;self.column=column;self.mark=mark
class Scanner:
	def __init__(self,loader=_C):
		self.loader=loader
		if self.loader is not _C and getattr(self.loader,'_scanner',_C)is _C:self.loader._scanner=self
		self.reset_scanner();self.first_time=_B;self.yaml_version=_C
	@property
	def flow_level(self):return len(self.flow_context)
	def reset_scanner(self):self.done=_B;self.flow_context=[];self.tokens=[];self.fetch_stream_start();self.tokens_taken=0;self.indent=-1;self.indents=[];self.allow_simple_key=_A;self.possible_simple_keys={}
	@property
	def reader(self):
		try:return self._scanner_reader
		except AttributeError:
			if hasattr(self.loader,_c):self._scanner_reader=self.loader.reader
			else:self._scanner_reader=self.loader._reader
			return self._scanner_reader
	@property
	def scanner_processing_version(self):
		if hasattr(self.loader,_c):return self.loader.resolver.processing_version
		return self.loader.processing_version
	def check_token(self,*choices):
		while self.need_more_tokens():self.fetch_more_tokens()
		if bool(self.tokens):
			if not choices:return _A
			for choice in choices:
				if isinstance(self.tokens[0],choice):return _A
		return _B
	def peek_token(self):
		while self.need_more_tokens():self.fetch_more_tokens()
		if bool(self.tokens):return self.tokens[0]
	def get_token(self):
		while self.need_more_tokens():self.fetch_more_tokens()
		if bool(self.tokens):self.tokens_taken+=1;return self.tokens.pop(0)
	def need_more_tokens(self):
		if self.done:return _B
		if not self.tokens:return _A
		self.stale_possible_simple_keys()
		if self.next_possible_simple_key()==self.tokens_taken:return _A
		return _B
	def fetch_comment(self,comment):raise NotImplementedError
	def fetch_more_tokens(self):
		comment=self.scan_to_next_token()
		if comment is not _C:return self.fetch_comment(comment)
		self.stale_possible_simple_keys();self.unwind_indent(self.reader.column);ch=self.reader.peek()
		if ch==_I:return self.fetch_stream_end()
		if ch=='%'and self.check_directive():return self.fetch_directive()
		if ch==_N and self.check_document_start():return self.fetch_document_start()
		if ch=='.'and self.check_document_end():return self.fetch_document_end()
		if ch=='[':return self.fetch_flow_sequence_start()
		if ch=='{':return self.fetch_flow_mapping_start()
		if ch==']':return self.fetch_flow_sequence_end()
		if ch=='}':return self.fetch_flow_mapping_end()
		if ch==',':return self.fetch_flow_entry()
		if ch==_N and self.check_block_entry():return self.fetch_block_entry()
		if ch=='?'and self.check_key():return self.fetch_key()
		if ch==_O and self.check_value():return self.fetch_value()
		if ch=='*':return self.fetch_alias()
		if ch=='&':return self.fetch_anchor()
		if ch==_H:return self.fetch_tag()
		if ch=='|'and not self.flow_level:return self.fetch_literal()
		if ch==_R and not self.flow_level:return self.fetch_folded()
		if ch==_J:return self.fetch_single()
		if ch==_P:return self.fetch_double()
		if self.check_plain():return self.fetch_plain()
		raise ScannerError('while scanning for the next token',_C,'found character %r that cannot start any token'%utf8(ch),self.reader.get_mark())
	def next_possible_simple_key(self):
		min_token_number=_C
		for level in self.possible_simple_keys:
			key=self.possible_simple_keys[level]
			if min_token_number is _C or key.token_number<min_token_number:min_token_number=key.token_number
		return min_token_number
	def stale_possible_simple_keys(self):
		for level in list(self.possible_simple_keys):
			key=self.possible_simple_keys[level]
			if key.line!=self.reader.line or self.reader.index-key.index>1024:
				if key.required:raise ScannerError(_d,key.mark,_e,self.reader.get_mark())
				del self.possible_simple_keys[level]
	def save_possible_simple_key(self):
		required=not self.flow_level and self.indent==self.reader.column
		if self.allow_simple_key:self.remove_possible_simple_key();token_number=self.tokens_taken+len(self.tokens);key=SimpleKey(token_number,required,self.reader.index,self.reader.line,self.reader.column,self.reader.get_mark());self.possible_simple_keys[self.flow_level]=key
	def remove_possible_simple_key(self):
		if self.flow_level in self.possible_simple_keys:
			key=self.possible_simple_keys[self.flow_level]
			if key.required:raise ScannerError(_d,key.mark,_e,self.reader.get_mark())
			del self.possible_simple_keys[self.flow_level]
	def unwind_indent(self,column):
		if bool(self.flow_level):return
		while self.indent>column:mark=self.reader.get_mark();self.indent=self.indents.pop();self.tokens.append(BlockEndToken(mark,mark))
	def add_indent(self,column):
		if self.indent<column:self.indents.append(self.indent);self.indent=column;return _A
		return _B
	def fetch_stream_start(self):mark=self.reader.get_mark();self.tokens.append(StreamStartToken(mark,mark,encoding=self.reader.encoding))
	def fetch_stream_end(self):self.unwind_indent(-1);self.remove_possible_simple_key();self.allow_simple_key=_B;self.possible_simple_keys={};mark=self.reader.get_mark();self.tokens.append(StreamEndToken(mark,mark));self.done=_A
	def fetch_directive(self):self.unwind_indent(-1);self.remove_possible_simple_key();self.allow_simple_key=_B;self.tokens.append(self.scan_directive())
	def fetch_document_start(self):self.fetch_document_indicator(DocumentStartToken)
	def fetch_document_end(self):self.fetch_document_indicator(DocumentEndToken)
	def fetch_document_indicator(self,TokenClass):self.unwind_indent(-1);self.remove_possible_simple_key();self.allow_simple_key=_B;start_mark=self.reader.get_mark();self.reader.forward(3);end_mark=self.reader.get_mark();self.tokens.append(TokenClass(start_mark,end_mark))
	def fetch_flow_sequence_start(self):self.fetch_flow_collection_start(FlowSequenceStartToken,to_push='[')
	def fetch_flow_mapping_start(self):self.fetch_flow_collection_start(FlowMappingStartToken,to_push='{')
	def fetch_flow_collection_start(self,TokenClass,to_push):self.save_possible_simple_key();self.flow_context.append(to_push);self.allow_simple_key=_A;start_mark=self.reader.get_mark();self.reader.forward();end_mark=self.reader.get_mark();self.tokens.append(TokenClass(start_mark,end_mark))
	def fetch_flow_sequence_end(self):self.fetch_flow_collection_end(FlowSequenceEndToken)
	def fetch_flow_mapping_end(self):self.fetch_flow_collection_end(FlowMappingEndToken)
	def fetch_flow_collection_end(self,TokenClass):
		self.remove_possible_simple_key()
		try:popped=self.flow_context.pop()
		except IndexError:pass
		self.allow_simple_key=_B;start_mark=self.reader.get_mark();self.reader.forward();end_mark=self.reader.get_mark();self.tokens.append(TokenClass(start_mark,end_mark))
	def fetch_flow_entry(self):self.allow_simple_key=_A;self.remove_possible_simple_key();start_mark=self.reader.get_mark();self.reader.forward();end_mark=self.reader.get_mark();self.tokens.append(FlowEntryToken(start_mark,end_mark))
	def fetch_block_entry(self):
		if not self.flow_level:
			if not self.allow_simple_key:raise ScannerError(_C,_C,'sequence entries are not allowed here',self.reader.get_mark())
			if self.add_indent(self.reader.column):mark=self.reader.get_mark();self.tokens.append(BlockSequenceStartToken(mark,mark))
		else:0
		self.allow_simple_key=_A;self.remove_possible_simple_key();start_mark=self.reader.get_mark();self.reader.forward();end_mark=self.reader.get_mark();self.tokens.append(BlockEntryToken(start_mark,end_mark))
	def fetch_key(self):
		if not self.flow_level:
			if not self.allow_simple_key:raise ScannerError(_C,_C,'mapping keys are not allowed here',self.reader.get_mark())
			if self.add_indent(self.reader.column):mark=self.reader.get_mark();self.tokens.append(BlockMappingStartToken(mark,mark))
		self.allow_simple_key=not self.flow_level;self.remove_possible_simple_key();start_mark=self.reader.get_mark();self.reader.forward();end_mark=self.reader.get_mark();self.tokens.append(KeyToken(start_mark,end_mark))
	def fetch_value(self):
		if self.flow_level in self.possible_simple_keys:
			key=self.possible_simple_keys[self.flow_level];del self.possible_simple_keys[self.flow_level];self.tokens.insert(key.token_number-self.tokens_taken,KeyToken(key.mark,key.mark))
			if not self.flow_level:
				if self.add_indent(key.column):self.tokens.insert(key.token_number-self.tokens_taken,BlockMappingStartToken(key.mark,key.mark))
			self.allow_simple_key=_B
		else:
			if not self.flow_level:
				if not self.allow_simple_key:raise ScannerError(_C,_C,'mapping values are not allowed here',self.reader.get_mark())
			if not self.flow_level:
				if self.add_indent(self.reader.column):mark=self.reader.get_mark();self.tokens.append(BlockMappingStartToken(mark,mark))
			self.allow_simple_key=not self.flow_level;self.remove_possible_simple_key()
		start_mark=self.reader.get_mark();self.reader.forward();end_mark=self.reader.get_mark();self.tokens.append(ValueToken(start_mark,end_mark))
	def fetch_alias(self):self.save_possible_simple_key();self.allow_simple_key=_B;self.tokens.append(self.scan_anchor(AliasToken))
	def fetch_anchor(self):self.save_possible_simple_key();self.allow_simple_key=_B;self.tokens.append(self.scan_anchor(AnchorToken))
	def fetch_tag(self):self.save_possible_simple_key();self.allow_simple_key=_B;self.tokens.append(self.scan_tag())
	def fetch_literal(self):self.fetch_block_scalar(style='|')
	def fetch_folded(self):self.fetch_block_scalar(style=_R)
	def fetch_block_scalar(self,style):self.allow_simple_key=_A;self.remove_possible_simple_key();self.tokens.append(self.scan_block_scalar(style))
	def fetch_single(self):self.fetch_flow_scalar(style=_J)
	def fetch_double(self):self.fetch_flow_scalar(style=_P)
	def fetch_flow_scalar(self,style):self.save_possible_simple_key();self.allow_simple_key=_B;self.tokens.append(self.scan_flow_scalar(style))
	def fetch_plain(self):self.save_possible_simple_key();self.allow_simple_key=_B;self.tokens.append(self.scan_plain())
	def check_directive(self):
		if self.reader.column==0:return _A
		return _C
	def check_document_start(self):
		if self.reader.column==0:
			if self.reader.prefix(3)==_S and self.reader.peek(3)in _THE_END_SPACE_TAB:return _A
		return _C
	def check_document_end(self):
		if self.reader.column==0:
			if self.reader.prefix(3)==_T and self.reader.peek(3)in _THE_END_SPACE_TAB:return _A
		return _C
	def check_block_entry(self):return self.reader.peek(1)in _THE_END_SPACE_TAB
	def check_key(self):
		if bool(self.flow_level):return _A
		return self.reader.peek(1)in _THE_END_SPACE_TAB
	def check_value(self):
		if self.scanner_processing_version==(1,1):
			if bool(self.flow_level):return _A
		elif bool(self.flow_level):
			if self.flow_context[-1]=='[':
				if self.reader.peek(1)not in _THE_END_SPACE_TAB:return _B
			elif self.tokens and isinstance(self.tokens[-1],ValueToken):
				if self.reader.peek(1)not in _THE_END_SPACE_TAB:return _B
			return _A
		return self.reader.peek(1)in _THE_END_SPACE_TAB
	def check_plain(self):
		B='?:';A='\x00 \t\r\n\x85\u2028\u2029-?:,[]{}#&*!|>\'"%@`';srp=self.reader.peek;ch=srp()
		if self.scanner_processing_version==(1,1):return ch not in A or srp(1)not in _THE_END_SPACE_TAB and(ch==_N or not self.flow_level and ch in B)
		if ch not in A:return _A
		ch1=srp(1)
		if ch==_N and ch1 not in _THE_END_SPACE_TAB:return _A
		if ch==_O and bool(self.flow_level)and ch1 not in _SPACE_TAB:return _A
		return srp(1)not in _THE_END_SPACE_TAB and(ch==_N or not self.flow_level and ch in B)
	def scan_to_next_token(self):
		srp=self.reader.peek;srf=self.reader.forward
		if self.reader.index==0 and srp()==_f:srf()
		found=_B;_the_end=_THE_END
		while not found:
			while srp()==_D:srf()
			if srp()==_F:
				while srp()not in _the_end:srf()
			if self.scan_line_break():
				if not self.flow_level:self.allow_simple_key=_A
			else:found=_A
		return _C
	def scan_directive(self):
		srp=self.reader.peek;srf=self.reader.forward;start_mark=self.reader.get_mark();srf();name=self.scan_directive_name(start_mark);value=_C
		if name=='YAML':value=self.scan_yaml_directive_value(start_mark);end_mark=self.reader.get_mark()
		elif name=='TAG':value=self.scan_tag_directive_value(start_mark);end_mark=self.reader.get_mark()
		else:
			end_mark=self.reader.get_mark()
			while srp()not in _THE_END:srf()
		self.scan_directive_ignored_line(start_mark);return DirectiveToken(name,value,start_mark,end_mark)
	def scan_directive_name(self,start_mark):
		length=0;srp=self.reader.peek;ch=srp(length)
		while _K<=ch<=_Q or'A'<=ch<='Z'or _U<=ch<='z'or ch in'-_:.':length+=1;ch=srp(length)
		if not length:raise ScannerError(_G,start_mark,_V%utf8(ch),self.reader.get_mark())
		value=self.reader.prefix(length);self.reader.forward(length);ch=srp()
		if ch not in _L:raise ScannerError(_G,start_mark,_V%utf8(ch),self.reader.get_mark())
		return value
	def scan_yaml_directive_value(self,start_mark):
		srp=self.reader.peek;srf=self.reader.forward
		while srp()==_D:srf()
		major=self.scan_yaml_directive_number(start_mark)
		if srp()!='.':raise ScannerError(_G,start_mark,"expected a digit or '.', but found %r"%utf8(srp()),self.reader.get_mark())
		srf();minor=self.scan_yaml_directive_number(start_mark)
		if srp()not in _L:raise ScannerError(_G,start_mark,"expected a digit or ' ', but found %r"%utf8(srp()),self.reader.get_mark())
		self.yaml_version=major,minor;return self.yaml_version
	def scan_yaml_directive_number(self,start_mark):
		srp=self.reader.peek;srf=self.reader.forward;ch=srp()
		if not _K<=ch<=_Q:raise ScannerError(_G,start_mark,'expected a digit, but found %r'%utf8(ch),self.reader.get_mark())
		length=0
		while _K<=srp(length)<=_Q:length+=1
		value=int(self.reader.prefix(length));srf(length);return value
	def scan_tag_directive_value(self,start_mark):
		srp=self.reader.peek;srf=self.reader.forward
		while srp()==_D:srf()
		handle=self.scan_tag_directive_handle(start_mark)
		while srp()==_D:srf()
		prefix=self.scan_tag_directive_prefix(start_mark);return handle,prefix
	def scan_tag_directive_handle(self,start_mark):
		value=self.scan_tag_handle(_g,start_mark);ch=self.reader.peek()
		if ch!=_D:raise ScannerError(_G,start_mark,_Z%utf8(ch),self.reader.get_mark())
		return value
	def scan_tag_directive_prefix(self,start_mark):
		value=self.scan_tag_uri(_g,start_mark);ch=self.reader.peek()
		if ch not in _L:raise ScannerError(_G,start_mark,_Z%utf8(ch),self.reader.get_mark())
		return value
	def scan_directive_ignored_line(self,start_mark):
		srp=self.reader.peek;srf=self.reader.forward
		while srp()==_D:srf()
		if srp()==_F:
			while srp()not in _THE_END:srf()
		ch=srp()
		if ch not in _THE_END:raise ScannerError(_G,start_mark,_h%utf8(ch),self.reader.get_mark())
		self.scan_line_break()
	def scan_anchor(self,TokenClass):
		A='while scanning an %s';srp=self.reader.peek;start_mark=self.reader.get_mark();indicator=srp()
		if indicator=='*':name='alias'
		else:name='anchor'
		self.reader.forward();length=0;ch=srp(length)
		while check_anchorname_char(ch):length+=1;ch=srp(length)
		if not length:raise ScannerError(A%(name,),start_mark,_V%utf8(ch),self.reader.get_mark())
		value=self.reader.prefix(length);self.reader.forward(length)
		if ch not in'\x00 \t\r\n\x85\u2028\u2029?:,[]{}%@`':raise ScannerError(A%(name,),start_mark,_V%utf8(ch),self.reader.get_mark())
		end_mark=self.reader.get_mark();return TokenClass(value,start_mark,end_mark)
	def scan_tag(self):
		A='tag';srp=self.reader.peek;start_mark=self.reader.get_mark();ch=srp(1)
		if ch=='<':
			handle=_C;self.reader.forward(2);suffix=self.scan_tag_uri(A,start_mark)
			if srp()!=_R:raise ScannerError('while parsing a tag',start_mark,"expected '>', but found %r"%utf8(srp()),self.reader.get_mark())
			self.reader.forward()
		elif ch in _THE_END_SPACE_TAB:handle=_C;suffix=_H;self.reader.forward()
		else:
			length=1;use_handle=_B
			while ch not in _L:
				if ch==_H:use_handle=_A;break
				length+=1;ch=srp(length)
			handle=_H
			if use_handle:handle=self.scan_tag_handle(A,start_mark)
			else:handle=_H;self.reader.forward()
			suffix=self.scan_tag_uri(A,start_mark)
		ch=srp()
		if ch not in _L:raise ScannerError('while scanning a tag',start_mark,_Z%utf8(ch),self.reader.get_mark())
		value=handle,suffix;end_mark=self.reader.get_mark();return TagToken(value,start_mark,end_mark)
	def scan_block_scalar(self,style,rt=_B):
		A='|>';srp=self.reader.peek
		if style==_R:folded=_A
		else:folded=_B
		chunks=[];start_mark=self.reader.get_mark();self.reader.forward();chomping,increment=self.scan_block_scalar_indicators(start_mark);block_scalar_comment=self.scan_block_scalar_ignored_line(start_mark);min_indent=self.indent+1
		if increment is _C:
			if min_indent<1 and(style not in A or self.scanner_processing_version==(1,1)and getattr(self.loader,'top_level_block_style_scalar_no_indent_error_1_1',_B)):min_indent=1
			breaks,max_indent,end_mark=self.scan_block_scalar_indentation();indent=max(min_indent,max_indent)
		else:
			if min_indent<1:min_indent=1
			indent=min_indent+increment-1;breaks,end_mark=self.scan_block_scalar_breaks(indent)
		line_break=''
		while self.reader.column==indent and srp()!=_I:
			chunks.extend(breaks);leading_non_space=srp()not in _M;length=0
			while srp(length)not in _THE_END:length+=1
			chunks.append(self.reader.prefix(length));self.reader.forward(length);line_break=self.scan_line_break();breaks,end_mark=self.scan_block_scalar_breaks(indent)
			if style in A and min_indent==0:
				if self.check_document_start()or self.check_document_end():break
			if self.reader.column==indent and srp()!=_I:
				if rt and folded and line_break==_E:chunks.append(_i)
				if folded and line_break==_E and leading_non_space and srp()not in _M:
					if not breaks:chunks.append(_D)
				else:chunks.append(line_break)
			else:break
		trailing=[]
		if chomping in[_C,_A]:chunks.append(line_break)
		if chomping is _A:chunks.extend(breaks)
		elif chomping in[_C,_B]:trailing.extend(breaks)
		token=ScalarToken(''.join(chunks),_B,start_mark,end_mark,style)
		if block_scalar_comment is not _C:token.add_pre_comments([block_scalar_comment])
		if len(trailing)>0:
			comment=self.scan_to_next_token()
			while comment:trailing.append(_D*comment[1].column+comment[0]);comment=self.scan_to_next_token()
			comment_end_mark=self.reader.get_mark();comment=CommentToken(''.join(trailing),end_mark,comment_end_mark);token.add_post_comment(comment)
		return token
	def scan_block_scalar_indicators(self,start_mark):
		D='expected indentation indicator in the range 1-9, but found 0';C='0123456789';B='+';A='+-';srp=self.reader.peek;chomping=_C;increment=_C;ch=srp()
		if ch in A:
			if ch==B:chomping=_A
			else:chomping=_B
			self.reader.forward();ch=srp()
			if ch in C:
				increment=int(ch)
				if increment==0:raise ScannerError(_W,start_mark,D,self.reader.get_mark())
				self.reader.forward()
		elif ch in C:
			increment=int(ch)
			if increment==0:raise ScannerError(_W,start_mark,D,self.reader.get_mark())
			self.reader.forward();ch=srp()
			if ch in A:
				if ch==B:chomping=_A
				else:chomping=_B
				self.reader.forward()
		ch=srp()
		if ch not in _L:raise ScannerError(_W,start_mark,'expected chomping or indentation indicators, but found %r'%utf8(ch),self.reader.get_mark())
		return chomping,increment
	def scan_block_scalar_ignored_line(self,start_mark):
		srp=self.reader.peek;srf=self.reader.forward;prefix='';comment=_C
		while srp()==_D:prefix+=srp();srf()
		if srp()==_F:
			comment=prefix
			while srp()not in _THE_END:comment+=srp();srf()
		ch=srp()
		if ch not in _THE_END:raise ScannerError(_W,start_mark,_h%utf8(ch),self.reader.get_mark())
		self.scan_line_break();return comment
	def scan_block_scalar_indentation(self):
		srp=self.reader.peek;srf=self.reader.forward;chunks=[];max_indent=0;end_mark=self.reader.get_mark()
		while srp()in _j:
			if srp()!=_D:chunks.append(self.scan_line_break());end_mark=self.reader.get_mark()
			else:
				srf()
				if self.reader.column>max_indent:max_indent=self.reader.column
		return chunks,max_indent,end_mark
	def scan_block_scalar_breaks(self,indent):
		chunks=[];srp=self.reader.peek;srf=self.reader.forward;end_mark=self.reader.get_mark()
		while self.reader.column<indent and srp()==_D:srf()
		while srp()in _X:
			chunks.append(self.scan_line_break());end_mark=self.reader.get_mark()
			while self.reader.column<indent and srp()==_D:srf()
		return chunks,end_mark
	def scan_flow_scalar(self,style):
		if style==_P:double=_A
		else:double=_B
		srp=self.reader.peek;chunks=[];start_mark=self.reader.get_mark();quote=srp();self.reader.forward();chunks.extend(self.scan_flow_scalar_non_spaces(double,start_mark))
		while srp()!=quote:chunks.extend(self.scan_flow_scalar_spaces(double,start_mark));chunks.extend(self.scan_flow_scalar_non_spaces(double,start_mark))
		self.reader.forward();end_mark=self.reader.get_mark();return ScalarToken(''.join(chunks),_B,start_mark,end_mark,style)
	ESCAPE_REPLACEMENTS={_K:_I,_U:_i,'b':'\x08','t':_a,_a:_a,'n':_E,'v':'\x0b','f':'\x0c','r':'\r','e':'\x1b',_D:_D,_P:_P,'/':'/',_b:_b,'N':'\x85','_':'\xa0','L':'\u2028','P':'\u2029'};ESCAPE_CODES={'x':2,'u':4,'U':8}
	def scan_flow_scalar_non_spaces(self,double,start_mark):
		A='while scanning a double-quoted scalar';chunks=[];srp=self.reader.peek;srf=self.reader.forward
		while _A:
			length=0
			while srp(length)not in' \n\'"\\\x00\t\r\x85\u2028\u2029':length+=1
			if length!=0:chunks.append(self.reader.prefix(length));srf(length)
			ch=srp()
			if not double and ch==_J and srp(1)==_J:chunks.append(_J);srf(2)
			elif double and ch==_J or not double and ch in'"\\':chunks.append(ch);srf()
			elif double and ch==_b:
				srf();ch=srp()
				if ch in self.ESCAPE_REPLACEMENTS:chunks.append(self.ESCAPE_REPLACEMENTS[ch]);srf()
				elif ch in self.ESCAPE_CODES:
					length=self.ESCAPE_CODES[ch];srf()
					for k in range(length):
						if srp(k)not in _k:raise ScannerError(A,start_mark,'expected escape sequence of %d hexdecimal numbers, but found %r'%(length,utf8(srp(k))),self.reader.get_mark())
					code=int(self.reader.prefix(length),16);chunks.append(unichr(code));srf(length)
				elif ch in'\n\r\x85\u2028\u2029':self.scan_line_break();chunks.extend(self.scan_flow_scalar_breaks(double,start_mark))
				else:raise ScannerError(A,start_mark,'found unknown escape character %r'%utf8(ch),self.reader.get_mark())
			else:return chunks
	def scan_flow_scalar_spaces(self,double,start_mark):
		srp=self.reader.peek;chunks=[];length=0
		while srp(length)in _M:length+=1
		whitespaces=self.reader.prefix(length);self.reader.forward(length);ch=srp()
		if ch==_I:raise ScannerError(_l,start_mark,'found unexpected end of stream',self.reader.get_mark())
		elif ch in _X:
			line_break=self.scan_line_break();breaks=self.scan_flow_scalar_breaks(double,start_mark)
			if line_break!=_E:chunks.append(line_break)
			elif not breaks:chunks.append(_D)
			chunks.extend(breaks)
		else:chunks.append(whitespaces)
		return chunks
	def scan_flow_scalar_breaks(self,double,start_mark):
		chunks=[];srp=self.reader.peek;srf=self.reader.forward
		while _A:
			prefix=self.reader.prefix(3)
			if(prefix==_S or prefix==_T)and srp(3)in _THE_END_SPACE_TAB:raise ScannerError(_l,start_mark,'found unexpected document separator',self.reader.get_mark())
			while srp()in _M:srf()
			if srp()in _X:chunks.append(self.scan_line_break())
			else:return chunks
	def scan_plain(self):
		srp=self.reader.peek;srf=self.reader.forward;chunks=[];start_mark=self.reader.get_mark();end_mark=start_mark;indent=self.indent+1;spaces=[]
		while _A:
			length=0
			if srp()==_F:break
			while _A:
				ch=srp(length)
				if ch==_O and srp(length+1)not in _THE_END_SPACE_TAB:0
				elif ch=='?'and self.scanner_processing_version!=(1,1):0
				elif ch in _THE_END_SPACE_TAB or not self.flow_level and ch==_O and srp(length+1)in _THE_END_SPACE_TAB or self.flow_level and ch in',:?[]{}':break
				length+=1
			if self.flow_level and ch==_O and srp(length+1)not in'\x00 \t\r\n\x85\u2028\u2029,[]{}':srf(length);raise ScannerError('while scanning a plain scalar',start_mark,"found unexpected ':'",self.reader.get_mark(),'Please check http://pyyaml.org/wiki/YAMLColonInFlowContext for details.')
			if length==0:break
			self.allow_simple_key=_B;chunks.extend(spaces);chunks.append(self.reader.prefix(length));srf(length);end_mark=self.reader.get_mark();spaces=self.scan_plain_spaces(indent,start_mark)
			if not spaces or srp()==_F or not self.flow_level and self.reader.column<indent:break
		token=ScalarToken(''.join(chunks),_A,start_mark,end_mark)
		if spaces and spaces[0]==_E:comment=CommentToken(''.join(spaces)+_E,start_mark,end_mark);token.add_post_comment(comment)
		return token
	def scan_plain_spaces(self,indent,start_mark):
		srp=self.reader.peek;srf=self.reader.forward;chunks=[];length=0
		while srp(length)in _D:length+=1
		whitespaces=self.reader.prefix(length);self.reader.forward(length);ch=srp()
		if ch in _X:
			line_break=self.scan_line_break();self.allow_simple_key=_A;prefix=self.reader.prefix(3)
			if(prefix==_S or prefix==_T)and srp(3)in _THE_END_SPACE_TAB:return
			breaks=[]
			while srp()in _j:
				if srp()==_D:srf()
				else:
					breaks.append(self.scan_line_break());prefix=self.reader.prefix(3)
					if(prefix==_S or prefix==_T)and srp(3)in _THE_END_SPACE_TAB:return
			if line_break!=_E:chunks.append(line_break)
			elif not breaks:chunks.append(_D)
			chunks.extend(breaks)
		elif whitespaces:chunks.append(whitespaces)
		return chunks
	def scan_tag_handle(self,name,start_mark):
		A="expected '!', but found %r";srp=self.reader.peek;ch=srp()
		if ch!=_H:raise ScannerError(_Y%(name,),start_mark,A%utf8(ch),self.reader.get_mark())
		length=1;ch=srp(length)
		if ch!=_D:
			while _K<=ch<=_Q or'A'<=ch<='Z'or _U<=ch<='z'or ch in'-_':length+=1;ch=srp(length)
			if ch!=_H:self.reader.forward(length);raise ScannerError(_Y%(name,),start_mark,A%utf8(ch),self.reader.get_mark())
			length+=1
		value=self.reader.prefix(length);self.reader.forward(length);return value
	def scan_tag_uri(self,name,start_mark):
		srp=self.reader.peek;chunks=[];length=0;ch=srp(length)
		while _K<=ch<=_Q or'A'<=ch<='Z'or _U<=ch<='z'or ch in"-;/?:@&=+$,_.!~*'()[]%"or self.scanner_processing_version>(1,1)and ch==_F:
			if ch=='%':chunks.append(self.reader.prefix(length));self.reader.forward(length);length=0;chunks.append(self.scan_uri_escapes(name,start_mark))
			else:length+=1
			ch=srp(length)
		if length!=0:chunks.append(self.reader.prefix(length));self.reader.forward(length);length=0
		if not chunks:raise ScannerError('while parsing a %s'%(name,),start_mark,'expected URI, but found %r'%utf8(ch),self.reader.get_mark())
		return ''.join(chunks)
	def scan_uri_escapes(self,name,start_mark):
		A='utf-8';srp=self.reader.peek;srf=self.reader.forward;code_bytes=[];mark=self.reader.get_mark()
		while srp()=='%':
			srf()
			for k in range(2):
				if srp(k)not in _k:raise ScannerError(_Y%(name,),start_mark,'expected URI escape sequence of 2 hexdecimal numbers, but found %r'%utf8(srp(k)),self.reader.get_mark())
			if PY3:code_bytes.append(int(self.reader.prefix(2),16))
			else:code_bytes.append(chr(int(self.reader.prefix(2),16)))
			srf(2)
		try:
			if PY3:value=bytes(code_bytes).decode(A)
			else:value=unicode(b''.join(code_bytes),A)
		except UnicodeDecodeError as exc:raise ScannerError(_Y%(name,),start_mark,str(exc),mark)
		return value
	def scan_line_break(self):
		ch=self.reader.peek()
		if ch in _m:
			if self.reader.prefix(2)==_n:self.reader.forward(2)
			else:self.reader.forward()
			return _E
		elif ch in _o:self.reader.forward();return ch
		return''
class RoundTripScanner(Scanner):
	def check_token(self,*choices):
		while self.need_more_tokens():self.fetch_more_tokens()
		self._gather_comments()
		if bool(self.tokens):
			if not choices:return _A
			for choice in choices:
				if isinstance(self.tokens[0],choice):return _A
		return _B
	def peek_token(self):
		while self.need_more_tokens():self.fetch_more_tokens()
		self._gather_comments()
		if bool(self.tokens):return self.tokens[0]
		return _C
	def _gather_comments(self):
		comments=[]
		if not self.tokens:return comments
		if isinstance(self.tokens[0],CommentToken):comment=self.tokens.pop(0);self.tokens_taken+=1;comments.append(comment)
		while self.need_more_tokens():
			self.fetch_more_tokens()
			if not self.tokens:return comments
			if isinstance(self.tokens[0],CommentToken):self.tokens_taken+=1;comment=self.tokens.pop(0);comments.append(comment)
		if len(comments)>=1:self.tokens[0].add_pre_comments(comments)
		if not self.done and len(self.tokens)<2:self.fetch_more_tokens()
	def get_token(self):
		while self.need_more_tokens():self.fetch_more_tokens()
		self._gather_comments()
		if bool(self.tokens):
			if len(self.tokens)>1 and isinstance(self.tokens[0],(ScalarToken,ValueToken,FlowSequenceEndToken,FlowMappingEndToken))and isinstance(self.tokens[1],CommentToken)and self.tokens[0].end_mark.line==self.tokens[1].start_mark.line:
				self.tokens_taken+=1;c=self.tokens.pop(1);self.fetch_more_tokens()
				while len(self.tokens)>1 and isinstance(self.tokens[1],CommentToken):self.tokens_taken+=1;c1=self.tokens.pop(1);c.value=c.value+_D*c1.start_mark.column+c1.value;self.fetch_more_tokens()
				self.tokens[0].add_post_comment(c)
			elif len(self.tokens)>1 and isinstance(self.tokens[0],ScalarToken)and isinstance(self.tokens[1],CommentToken)and self.tokens[0].end_mark.line!=self.tokens[1].start_mark.line:
				self.tokens_taken+=1;c=self.tokens.pop(1);c.value=_E*(c.start_mark.line-self.tokens[0].end_mark.line)+_D*c.start_mark.column+c.value;self.tokens[0].add_post_comment(c);self.fetch_more_tokens()
				while len(self.tokens)>1 and isinstance(self.tokens[1],CommentToken):self.tokens_taken+=1;c1=self.tokens.pop(1);c.value=c.value+_D*c1.start_mark.column+c1.value;self.fetch_more_tokens()
			self.tokens_taken+=1;return self.tokens.pop(0)
		return _C
	def fetch_comment(self,comment):
		value,start_mark,end_mark=comment
		while value and value[-1]==_D:value=value[:-1]
		self.tokens.append(CommentToken(value,start_mark,end_mark))
	def scan_to_next_token(self):
		srp=self.reader.peek;srf=self.reader.forward
		if self.reader.index==0 and srp()==_f:srf()
		found=_B
		while not found:
			while srp()==_D:srf()
			ch=srp()
			if ch==_F:
				start_mark=self.reader.get_mark();comment=ch;srf()
				while ch not in _THE_END:
					ch=srp()
					if ch==_I:comment+=_E;break
					comment+=ch;srf()
				ch=self.scan_line_break()
				while len(ch)>0:comment+=ch;ch=self.scan_line_break()
				end_mark=self.reader.get_mark()
				if not self.flow_level:self.allow_simple_key=_A
				return comment,start_mark,end_mark
			if bool(self.scan_line_break()):
				start_mark=self.reader.get_mark()
				if not self.flow_level:self.allow_simple_key=_A
				ch=srp()
				if ch==_E:
					start_mark=self.reader.get_mark();comment=''
					while ch:ch=self.scan_line_break(empty_line=_A);comment+=ch
					if srp()==_F:comment=comment.rsplit(_E,1)[0]+_E
					end_mark=self.reader.get_mark();return comment,start_mark,end_mark
			else:found=_A
		return _C
	def scan_line_break(self,empty_line=_B):
		ch=self.reader.peek()
		if ch in _m:
			if self.reader.prefix(2)==_n:self.reader.forward(2)
			else:self.reader.forward()
			return _E
		elif ch in _o:self.reader.forward();return ch
		elif empty_line and ch in'\t ':self.reader.forward();return ch
		return''
	def scan_block_scalar(self,style,rt=_A):return Scanner.scan_block_scalar(self,style,rt=rt)


from __future__ import absolute_import
_I='once'
_H='  in "%s", line %d, column %d'
_G='line'
_F='index'
_E='name'
_D='column'
_C=False
_B='\n'
_A=None
import warnings,textwrap
from .compat import utf8
if _C:from typing import Any,Dict,Optional,List,Text
__all__=['FileMark','StringMark','CommentMark','YAMLError','MarkedYAMLError','ReusedAnchorWarning','UnsafeLoaderWarning','MarkedYAMLWarning','MarkedYAMLFutureWarning']
class StreamMark:
	__slots__=_E,_F,_G,_D
	def __init__(A,name,index,line,column):A.name=name;A.index=index;A.line=line;A.column=column
	def __str__(A):B=_H%(A.name,A.line+1,A.column+1);return B
	def __eq__(A,other):
		B=other
		if A.line!=B.line or A.column!=B.column:return _C
		if A.name!=B.name or A.index!=B.index:return _C
		return True
	def __ne__(A,other):return not A.__eq__(other)
class FileMark(StreamMark):__slots__=()
class StringMark(StreamMark):
	__slots__=_E,_F,_G,_D,'buffer','pointer'
	def __init__(A,name,index,line,column,buffer,pointer):StreamMark.__init__(A,name,index,line,column);A.buffer=buffer;A.pointer=pointer
	def get_snippet(A,indent=4,max_length=75):
		L=' ';K=' ... ';J='\x00\r\n\x85\u2028\u2029';F=max_length;E=indent
		if A.buffer is _A:return _A
		D='';B=A.pointer
		while B>0 and A.buffer[B-1]not in J:
			B-=1
			if A.pointer-B>F/2-1:D=K;B+=5;break
		G='';C=A.pointer
		while C<len(A.buffer)and A.buffer[C]not in J:
			C+=1
			if C-A.pointer>F/2-1:G=K;C-=5;break
		I=utf8(A.buffer[B:C]);H='^';H='^ (line: {})'.format(A.line+1);return L*E+D+I+G+_B+L*(E+A.pointer-B+len(D))+H
	def __str__(A):
		B=A.get_snippet();C=_H%(A.name,A.line+1,A.column+1)
		if B is not _A:C+=':\n'+B
		return C
class CommentMark:
	__slots__=_D,
	def __init__(A,column):A.column=column
class YAMLError(Exception):0
class MarkedYAMLError(YAMLError):
	def __init__(A,context=_A,context_mark=_A,problem=_A,problem_mark=_A,note=_A,warn=_A):A.context=context;A.context_mark=context_mark;A.problem=problem;A.problem_mark=problem_mark;A.note=note
	def __str__(A):
		B=[]
		if A.context is not _A:B.append(A.context)
		if A.context_mark is not _A and(A.problem is _A or A.problem_mark is _A or A.context_mark.name!=A.problem_mark.name or A.context_mark.line!=A.problem_mark.line or A.context_mark.column!=A.problem_mark.column):B.append(str(A.context_mark))
		if A.problem is not _A:B.append(A.problem)
		if A.problem_mark is not _A:B.append(str(A.problem_mark))
		if A.note is not _A and A.note:C=textwrap.dedent(A.note);B.append(C)
		return _B.join(B)
class YAMLStreamError(Exception):0
class YAMLWarning(Warning):0
class MarkedYAMLWarning(YAMLWarning):
	def __init__(A,context=_A,context_mark=_A,problem=_A,problem_mark=_A,note=_A,warn=_A):A.context=context;A.context_mark=context_mark;A.problem=problem;A.problem_mark=problem_mark;A.note=note;A.warn=warn
	def __str__(A):
		B=[]
		if A.context is not _A:B.append(A.context)
		if A.context_mark is not _A and(A.problem is _A or A.problem_mark is _A or A.context_mark.name!=A.problem_mark.name or A.context_mark.line!=A.problem_mark.line or A.context_mark.column!=A.problem_mark.column):B.append(str(A.context_mark))
		if A.problem is not _A:B.append(A.problem)
		if A.problem_mark is not _A:B.append(str(A.problem_mark))
		if A.note is not _A and A.note:C=textwrap.dedent(A.note);B.append(C)
		if A.warn is not _A and A.warn:D=textwrap.dedent(A.warn);B.append(D)
		return _B.join(B)
class ReusedAnchorWarning(YAMLWarning):0
class UnsafeLoaderWarning(YAMLWarning):text="\nThe default 'Loader' for 'load(stream)' without further arguments can be unsafe.\nUse 'load(stream, Loader=ruamel.yaml.Loader)' explicitly if that is OK.\nAlternatively include the following in your code:\n\n  import warnings\n  warnings.simplefilter('ignore', ruamel.yaml.error.UnsafeLoaderWarning)\n\nIn most other cases you should consider using 'safe_load(stream)'"
warnings.simplefilter(_I,UnsafeLoaderWarning)
class MantissaNoDotYAML1_1Warning(YAMLWarning):
	def __init__(A,node,flt_str):A.node=node;A.flt=flt_str
	def __str__(A):B=A.node.start_mark.line;C=A.node.start_mark.column;return '\nIn YAML 1.1 floating point values should have a dot (\'.\') in their mantissa.\nSee the Floating-Point Language-Independent Type for YAML™ Version 1.1 specification\n( http://yaml.org/type/float.html ). This dot is not required for JSON nor for YAML 1.2\n\nCorrect your float: "{}" on line: {}, column: {}\n\nor alternatively include the following in your code:\n\n  import warnings\n  warnings.simplefilter(\'ignore\', ruamel.yaml.error.MantissaNoDotYAML1_1Warning)\n\n'.format(A.flt,B,C)
warnings.simplefilter(_I,MantissaNoDotYAML1_1Warning)
class YAMLFutureWarning(Warning):0
class MarkedYAMLFutureWarning(YAMLFutureWarning):
	def __init__(A,context=_A,context_mark=_A,problem=_A,problem_mark=_A,note=_A,warn=_A):A.context=context;A.context_mark=context_mark;A.problem=problem;A.problem_mark=problem_mark;A.note=note;A.warn=warn
	def __str__(A):
		B=[]
		if A.context is not _A:B.append(A.context)
		if A.context_mark is not _A and(A.problem is _A or A.problem_mark is _A or A.context_mark.name!=A.problem_mark.name or A.context_mark.line!=A.problem_mark.line or A.context_mark.column!=A.problem_mark.column):B.append(str(A.context_mark))
		if A.problem is not _A:B.append(A.problem)
		if A.problem_mark is not _A:B.append(str(A.problem_mark))
		if A.note is not _A and A.note:C=textwrap.dedent(A.note);B.append(C)
		if A.warn is not _A and A.warn:D=textwrap.dedent(A.warn);B.append(D)
		return _B.join(B)


_G=']\n'
_F=' = '
_E='\n'
_D=False
_C='['
_B='.'
_A=None
import datetime,re,sys
from decimal import Decimal
from .decoder import InlineTableDict
if sys.version_info>=(3,):unicode=str
def dump(o,f,encoder=_A):
	if not f.write:raise TypeError('You can only dump an object to a file descriptor')
	A=dumps(o,encoder=encoder);f.write(A);return A
def dumps(o,encoder=_A):
	C=encoder;A=''
	if C is _A:C=TomlEncoder(o.__class__)
	B,D=C.dump_sections(o,'');A+=B;G=[id(o)]
	while D:
		H=[id(A)for A in D]
		for K in G:
			if K in H:raise ValueError('Circular reference detected')
		G+=H;I=C.get_empty_table()
		for E in D:
			B,F=C.dump_sections(D[E],E)
			if B or not B and not F:
				if A and A[-2:]!='\n\n':A+=_E
				A+=_C+E+_G
				if B:A+=B
			for J in F:I[E+_B+J]=F[J]
		D=I
	return A
def _dump_str(v):
	G="'";F='\\';C='"'
	if sys.version_info<(3,)and hasattr(v,'decode')and isinstance(v,str):v=v.decode('utf-8')
	v='%r'%v
	if v[0]=='u':v=v[1:]
	D=v.startswith(G)
	if D or v.startswith(C):v=v[1:-1]
	if D:v=v.replace("\\'",G);v=v.replace(C,'\\"')
	v=v.split('\\x')
	while len(v)>1:
		A=-1
		if not v[0]:v=v[1:]
		v[0]=v[0].replace('\\\\',F);B=v[0][A]!=F
		while v[0][:A]and v[0][A]==F:B=not B;A-=1
		if B:E='x'
		else:E='u00'
		v=[v[0]+E+v[1]]+v[2:]
	return unicode(C+v[0]+C)
def _dump_float(v):return '{}'.format(v).replace('e+0','e+').replace('e-0','e-')
def _dump_time(v):
	A=v.utcoffset()
	if A is _A:return v.isoformat()
	return v.isoformat()[:-6]
class TomlEncoder:
	def __init__(A,_dict=dict,preserve=_D):A._dict=_dict;A.preserve=preserve;A.dump_funcs={str:_dump_str,unicode:_dump_str,list:A.dump_list,bool:lambda v:unicode(v).lower(),int:lambda v:v,float:_dump_float,Decimal:_dump_float,datetime.datetime:lambda v:v.isoformat().replace('+00:00','Z'),datetime.time:_dump_time,datetime.date:lambda v:v.isoformat()}
	def get_empty_table(A):return A._dict()
	def dump_list(B,v):
		A=_C
		for C in v:A+=' '+unicode(B.dump_value(C))+','
		A+=']';return A
	def dump_inline_table(B,section):
		A=section;C=''
		if isinstance(A,dict):
			D=[]
			for (E,F) in A.items():G=B.dump_inline_table(F);D.append(E+_F+G)
			C+='{ '+', '.join(D)+' }\n';return C
		else:return unicode(B.dump_value(A))
	def dump_value(B,v):
		A=B.dump_funcs.get(type(v))
		if A is _A and hasattr(v,'__iter__'):A=B.dump_funcs[list]
		return A(v)if A is not _A else B.dump_funcs[str](v)
	def dump_sections(C,o,sup):
		D=sup;F=''
		if D!=''and D[-1]!=_B:D+=_B
		M=C._dict();G=''
		for A in o:
			A=unicode(A);B=A
			if not re.match('^[A-Za-z0-9_-]+$',A):B=_dump_str(A)
			if not isinstance(o[A],dict):
				N=_D
				if isinstance(o[A],list):
					for L in o[A]:
						if isinstance(L,dict):N=True
				if N:
					for L in o[A]:
						H=_E;G+='[['+D+B+']]\n';I,J=C.dump_sections(L,D+B)
						if I:
							if I[0]==_C:H+=I
							else:G+=I
						while J:
							O=C._dict()
							for K in J:
								E,P=C.dump_sections(J[K],D+B+_B+K)
								if E:H+=_C+D+B+_B+K+_G;H+=E
								for E in P:O[K+_B+E]=P[E]
							J=O
						G+=H
				elif o[A]is not _A:F+=B+_F+unicode(C.dump_value(o[A]))+_E
			elif C.preserve and isinstance(o[A],InlineTableDict):F+=B+_F+C.dump_inline_table(o[A])
			else:M[B]=o[A]
		F+=G;return F,M
class TomlPreserveInlineDictEncoder(TomlEncoder):
	def __init__(A,_dict=dict):super(TomlPreserveInlineDictEncoder,A).__init__(_dict,True)
class TomlArraySeparatorEncoder(TomlEncoder):
	def __init__(B,_dict=dict,preserve=_D,separator=','):
		A=separator;super(TomlArraySeparatorEncoder,B).__init__(_dict,preserve)
		if A.strip()=='':A=','+A
		elif A.strip(' \t\n\r,'):raise ValueError('Invalid separator for arrays')
		B.separator=A
	def dump_list(D,v):
		B=[];C=_C
		for A in v:B.append(D.dump_value(A))
		while B!=[]:
			E=[]
			for A in B:
				if isinstance(A,list):
					for F in A:E.append(F)
				else:C+=' '+unicode(A)+D.separator
			B=E
		C+=']';return C
class TomlNumpyEncoder(TomlEncoder):
	def __init__(A,_dict=dict,preserve=_D):import numpy as B;super(TomlNumpyEncoder,A).__init__(_dict,preserve);A.dump_funcs[B.float16]=_dump_float;A.dump_funcs[B.float32]=_dump_float;A.dump_funcs[B.float64]=_dump_float;A.dump_funcs[B.int16]=A._dump_int;A.dump_funcs[B.int32]=A._dump_int;A.dump_funcs[B.int64]=A._dump_int
	def _dump_int(A,v):return '{}'.format(int(v))
class TomlPreserveCommentEncoder(TomlEncoder):
	def __init__(A,_dict=dict,preserve=_D):from dynaconf.vendor.toml.decoder import CommentValue as B;super(TomlPreserveCommentEncoder,A).__init__(_dict,preserve);A.dump_funcs[B]=lambda v:v.dump(A.dump_value)
class TomlPathlibEncoder(TomlEncoder):
	def _dump_pathlib_path(A,v):return _dump_str(str(v))
	def dump_value(A,v):
		if(3,4)<=sys.version_info:
			import pathlib as B
			if isinstance(v,B.PurePath):v=str(v)
		return super(TomlPathlibEncoder,A).dump_value(v)


from __future__ import absolute_import
_J='yaml_implicit_resolvers'
_I='typ'
_H='-+0123456789'
_G='tag:yaml.org,2002:int'
_F='-+0123456789.'
_E='tag:yaml.org,2002:float'
_D='tag:yaml.org,2002:bool'
_C=True
_B=False
_A=None
import re
if _B:from typing import Any,Dict,List,Union,Text,Optional;from .compat import VersionType
from .compat import string_types,_DEFAULT_YAML_VERSION
from .error import *
from .nodes import MappingNode,ScalarNode,SequenceNode
from .util import RegExp
__all__=['BaseResolver','Resolver','VersionedResolver']
implicit_resolvers=[([(1,2)],_D,RegExp('^(?:true|True|TRUE|false|False|FALSE)$',re.X),list('tTfF')),([(1,1)],_D,RegExp('^(?:y|Y|yes|Yes|YES|n|N|no|No|NO\n        |true|True|TRUE|false|False|FALSE\n        |on|On|ON|off|Off|OFF)$',re.X),list('yYnNtTfFoO')),([(1,2)],_E,RegExp('^(?:\n         [-+]?(?:[0-9][0-9_]*)\\.[0-9_]*(?:[eE][-+]?[0-9]+)?\n        |[-+]?(?:[0-9][0-9_]*)(?:[eE][-+]?[0-9]+)\n        |[-+]?\\.[0-9_]+(?:[eE][-+][0-9]+)?\n        |[-+]?\\.(?:inf|Inf|INF)\n        |\\.(?:nan|NaN|NAN))$',re.X),list(_F)),([(1,1)],_E,RegExp('^(?:\n         [-+]?(?:[0-9][0-9_]*)\\.[0-9_]*(?:[eE][-+]?[0-9]+)?\n        |[-+]?(?:[0-9][0-9_]*)(?:[eE][-+]?[0-9]+)\n        |\\.[0-9_]+(?:[eE][-+][0-9]+)?\n        |[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\\.[0-9_]*  # sexagesimal float\n        |[-+]?\\.(?:inf|Inf|INF)\n        |\\.(?:nan|NaN|NAN))$',re.X),list(_F)),([(1,2)],_G,RegExp('^(?:[-+]?0b[0-1_]+\n        |[-+]?0o?[0-7_]+\n        |[-+]?[0-9_]+\n        |[-+]?0x[0-9a-fA-F_]+)$',re.X),list(_H)),([(1,1)],_G,RegExp('^(?:[-+]?0b[0-1_]+\n        |[-+]?0?[0-7_]+\n        |[-+]?(?:0|[1-9][0-9_]*)\n        |[-+]?0x[0-9a-fA-F_]+\n        |[-+]?[1-9][0-9_]*(?::[0-5]?[0-9])+)$',re.X),list(_H)),([(1,2),(1,1)],'tag:yaml.org,2002:merge',RegExp('^(?:<<)$'),['<']),([(1,2),(1,1)],'tag:yaml.org,2002:null',RegExp('^(?: ~\n        |null|Null|NULL\n        | )$',re.X),['~','n','N','']),([(1,2),(1,1)],'tag:yaml.org,2002:timestamp',RegExp('^(?:[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]\n        |[0-9][0-9][0-9][0-9] -[0-9][0-9]? -[0-9][0-9]?\n        (?:[Tt]|[ \\t]+)[0-9][0-9]?\n        :[0-9][0-9] :[0-9][0-9] (?:\\.[0-9]*)?\n        (?:[ \\t]*(?:Z|[-+][0-9][0-9]?(?::[0-9][0-9])?))?)$',re.X),list('0123456789')),([(1,2),(1,1)],'tag:yaml.org,2002:value',RegExp('^(?:=)$'),['=']),([(1,2),(1,1)],'tag:yaml.org,2002:yaml',RegExp('^(?:!|&|\\*)$'),list('!&*'))]
class ResolverError(YAMLError):0
class BaseResolver:
	DEFAULT_SCALAR_TAG='tag:yaml.org,2002:str';DEFAULT_SEQUENCE_TAG='tag:yaml.org,2002:seq';DEFAULT_MAPPING_TAG='tag:yaml.org,2002:map';yaml_implicit_resolvers={};yaml_path_resolvers={}
	def __init__(self,loadumper=_A):
		self.loadumper=loadumper
		if self.loadumper is not _A and getattr(self.loadumper,'_resolver',_A)is _A:self.loadumper._resolver=self.loadumper
		self._loader_version=_A;self.resolver_exact_paths=[];self.resolver_prefix_paths=[]
	@property
	def parser(self):
		if self.loadumper is not _A:
			if hasattr(self.loadumper,_I):return self.loadumper.parser
			return self.loadumper._parser
		return _A
	@classmethod
	def add_implicit_resolver_base(cls,tag,regexp,first):
		if _J not in cls.__dict__:cls.yaml_implicit_resolvers=dict(((k,cls.yaml_implicit_resolvers[k][:])for k in cls.yaml_implicit_resolvers))
		if first is _A:first=[_A]
		for ch in first:cls.yaml_implicit_resolvers.setdefault(ch,[]).append((tag,regexp))
	@classmethod
	def add_implicit_resolver(cls,tag,regexp,first):
		if _J not in cls.__dict__:cls.yaml_implicit_resolvers=dict(((k,cls.yaml_implicit_resolvers[k][:])for k in cls.yaml_implicit_resolvers))
		if first is _A:first=[_A]
		for ch in first:cls.yaml_implicit_resolvers.setdefault(ch,[]).append((tag,regexp))
		implicit_resolvers.append(([(1,2),(1,1)],tag,regexp,first))
	@classmethod
	def add_path_resolver(cls,tag,path,kind=_A):
		if'yaml_path_resolvers'not in cls.__dict__:cls.yaml_path_resolvers=cls.yaml_path_resolvers.copy()
		new_path=[]
		for element in path:
			if isinstance(element,(list,tuple)):
				if len(element)==2:node_check,index_check=element
				elif len(element)==1:node_check=element[0];index_check=_C
				else:raise ResolverError('Invalid path element: %s'%(element,))
			else:node_check=_A;index_check=element
			if node_check is str:node_check=ScalarNode
			elif node_check is list:node_check=SequenceNode
			elif node_check is dict:node_check=MappingNode
			elif node_check not in[ScalarNode,SequenceNode,MappingNode]and not isinstance(node_check,string_types)and node_check is not _A:raise ResolverError('Invalid node checker: %s'%(node_check,))
			if not isinstance(index_check,(string_types,int))and index_check is not _A:raise ResolverError('Invalid index checker: %s'%(index_check,))
			new_path.append((node_check,index_check))
		if kind is str:kind=ScalarNode
		elif kind is list:kind=SequenceNode
		elif kind is dict:kind=MappingNode
		elif kind not in[ScalarNode,SequenceNode,MappingNode]and kind is not _A:raise ResolverError('Invalid node kind: %s'%(kind,))
		cls.yaml_path_resolvers[(tuple(new_path),kind)]=tag
	def descend_resolver(self,current_node,current_index):
		if not self.yaml_path_resolvers:return
		exact_paths={};prefix_paths=[]
		if current_node:
			depth=len(self.resolver_prefix_paths)
			for (path,kind) in self.resolver_prefix_paths[-1]:
				if self.check_resolver_prefix(depth,path,kind,current_node,current_index):
					if len(path)>depth:prefix_paths.append((path,kind))
					else:exact_paths[kind]=self.yaml_path_resolvers[(path,kind)]
		else:
			for (path,kind) in self.yaml_path_resolvers:
				if not path:exact_paths[kind]=self.yaml_path_resolvers[(path,kind)]
				else:prefix_paths.append((path,kind))
		self.resolver_exact_paths.append(exact_paths);self.resolver_prefix_paths.append(prefix_paths)
	def ascend_resolver(self):
		if not self.yaml_path_resolvers:return
		self.resolver_exact_paths.pop();self.resolver_prefix_paths.pop()
	def check_resolver_prefix(self,depth,path,kind,current_node,current_index):
		node_check,index_check=path[depth-1]
		if isinstance(node_check,string_types):
			if current_node.tag!=node_check:return _B
		elif node_check is not _A:
			if not isinstance(current_node,node_check):return _B
		if index_check is _C and current_index is not _A:return _B
		if(index_check is _B or index_check is _A)and current_index is _A:return _B
		if isinstance(index_check,string_types):
			if not(isinstance(current_index,ScalarNode)and index_check==current_index.value):return _B
		elif isinstance(index_check,int)and not isinstance(index_check,bool):
			if index_check!=current_index:return _B
		return _C
	def resolve(self,kind,value,implicit):
		if kind is ScalarNode and implicit[0]:
			if value=='':resolvers=self.yaml_implicit_resolvers.get('',[])
			else:resolvers=self.yaml_implicit_resolvers.get(value[0],[])
			resolvers+=self.yaml_implicit_resolvers.get(_A,[])
			for (tag,regexp) in resolvers:
				if regexp.match(value):return tag
			implicit=implicit[1]
		if bool(self.yaml_path_resolvers):
			exact_paths=self.resolver_exact_paths[-1]
			if kind in exact_paths:return exact_paths[kind]
			if _A in exact_paths:return exact_paths[_A]
		if kind is ScalarNode:return self.DEFAULT_SCALAR_TAG
		elif kind is SequenceNode:return self.DEFAULT_SEQUENCE_TAG
		elif kind is MappingNode:return self.DEFAULT_MAPPING_TAG
	@property
	def processing_version(self):return _A
class Resolver(BaseResolver):0
for ir in implicit_resolvers:
	if(1,2)in ir[0]:Resolver.add_implicit_resolver_base(*ir[1:])
class VersionedResolver(BaseResolver):
	def __init__(self,version=_A,loader=_A,loadumper=_A):
		if loader is _A and loadumper is not _A:loader=loadumper
		BaseResolver.__init__(self,loader);self._loader_version=self.get_loader_version(version);self._version_implicit_resolver={}
	def add_version_implicit_resolver(self,version,tag,regexp,first):
		if first is _A:first=[_A]
		impl_resolver=self._version_implicit_resolver.setdefault(version,{})
		for ch in first:impl_resolver.setdefault(ch,[]).append((tag,regexp))
	def get_loader_version(self,version):
		if version is _A or isinstance(version,tuple):return version
		if isinstance(version,list):return tuple(version)
		return tuple(map(int,version.split('.')))
	@property
	def versioned_resolver(self):
		version=self.processing_version
		if version not in self._version_implicit_resolver:
			for x in implicit_resolvers:
				if version in x[0]:self.add_version_implicit_resolver(version,x[1],x[2],x[3])
		return self._version_implicit_resolver[version]
	def resolve(self,kind,value,implicit):
		if kind is ScalarNode and implicit[0]:
			if value=='':resolvers=self.versioned_resolver.get('',[])
			else:resolvers=self.versioned_resolver.get(value[0],[])
			resolvers+=self.versioned_resolver.get(_A,[])
			for (tag,regexp) in resolvers:
				if regexp.match(value):return tag
			implicit=implicit[1]
		if bool(self.yaml_path_resolvers):
			exact_paths=self.resolver_exact_paths[-1]
			if kind in exact_paths:return exact_paths[kind]
			if _A in exact_paths:return exact_paths[_A]
		if kind is ScalarNode:return self.DEFAULT_SCALAR_TAG
		elif kind is SequenceNode:return self.DEFAULT_SEQUENCE_TAG
		elif kind is MappingNode:return self.DEFAULT_MAPPING_TAG
	@property
	def processing_version(self):
		try:version=self.loadumper._scanner.yaml_version
		except AttributeError:
			try:
				if hasattr(self.loadumper,_I):version=self.loadumper.version
				else:version=self.loadumper._serializer.use_version
			except AttributeError:version=_A
		if version is _A:
			version=self._loader_version
			if version is _A:version=_DEFAULT_YAML_VERSION
		return version


_L='stderr'
_K='stdout'
_J='stdin'
_I='buffer'
_H='ascii'
_G='win'
_F='utf-8'
_E='encoding'
_D='replace'
_C=True
_B=False
_A=None
import codecs,io,os,re,sys
from weakref import WeakKeyDictionary
CYGWIN=sys.platform.startswith('cygwin')
MSYS2=sys.platform.startswith(_G)and'GCC'in sys.version
APP_ENGINE='APPENGINE_RUNTIME'in os.environ and'Development/'in os.environ.get('SERVER_SOFTWARE','')
WIN=sys.platform.startswith(_G)and not APP_ENGINE and not MSYS2
DEFAULT_COLUMNS=80
auto_wrap_for_ansi=_A
colorama=_A
get_winterm_size=_A
_ansi_re=re.compile('\\033\\[[;?0-9]*[a-zA-Z]')
def get_filesystem_encoding():return sys.getfilesystemencoding()or sys.getdefaultencoding()
def _make_text_stream(stream,encoding,errors,force_readable=_B,force_writable=_B):
	C=stream;B=errors;A=encoding
	if A is _A:A=get_best_encoding(C)
	if B is _A:B=_D
	return _NonClosingTextIOWrapper(C,A,B,line_buffering=_C,force_readable=force_readable,force_writable=force_writable)
def is_ascii_encoding(encoding):
	try:return codecs.lookup(encoding).name==_H
	except LookupError:return _B
def get_best_encoding(stream):
	A=getattr(stream,_E,_A)or sys.getdefaultencoding()
	if is_ascii_encoding(A):return _F
	return A
class _NonClosingTextIOWrapper(io.TextIOWrapper):
	def __init__(B,stream,encoding,errors,force_readable=_B,force_writable=_B,**C):A=stream;B._stream=A=_FixupStream(A,force_readable,force_writable);super().__init__(A,encoding,errors,**C)
	def __del__(A):
		try:A.detach()
		except Exception:pass
	def isatty(A):return A._stream.isatty()
class _FixupStream:
	def __init__(A,stream,force_readable=_B,force_writable=_B):A._stream=stream;A._force_readable=force_readable;A._force_writable=force_writable
	def __getattr__(A,name):return getattr(A._stream,name)
	def read1(A,size):
		B=getattr(A._stream,'read1',_A)
		if B is not _A:return B(size)
		return A._stream.read(size)
	def readable(A):
		if A._force_readable:return _C
		B=getattr(A._stream,'readable',_A)
		if B is not _A:return B()
		try:A._stream.read(0)
		except Exception:return _B
		return _C
	def writable(A):
		if A._force_writable:return _C
		B=getattr(A._stream,'writable',_A)
		if B is not _A:return B()
		try:A._stream.write('')
		except Exception:
			try:A._stream.write(b'')
			except Exception:return _B
		return _C
	def seekable(A):
		B=getattr(A._stream,'seekable',_A)
		if B is not _A:return B()
		try:A._stream.seek(A._stream.tell())
		except Exception:return _B
		return _C
def is_bytes(x):return isinstance(x,(bytes,memoryview,bytearray))
def _is_binary_reader(stream,default=_B):
	try:return isinstance(stream.read(0),bytes)
	except Exception:return default
def _is_binary_writer(stream,default=_B):
	A=stream
	try:A.write(b'')
	except Exception:
		try:A.write('');return _B
		except Exception:pass
		return default
	return _C
def _find_binary_reader(stream):
	A=stream
	if _is_binary_reader(A,_B):return A
	B=getattr(A,_I,_A)
	if B is not _A and _is_binary_reader(B,_C):return B
def _find_binary_writer(stream):
	A=stream
	if _is_binary_writer(A,_B):return A
	B=getattr(A,_I,_A)
	if B is not _A and _is_binary_writer(B,_C):return B
def _stream_is_misconfigured(stream):return is_ascii_encoding(getattr(stream,_E,_A)or _H)
def _is_compat_stream_attr(stream,attr,value):A=value;B=getattr(stream,attr,_A);return B==A or A is _A and B is not _A
def _is_compatible_text_stream(stream,encoding,errors):A=stream;return _is_compat_stream_attr(A,_E,encoding)and _is_compat_stream_attr(A,'errors',errors)
def _force_correct_text_stream(text_stream,encoding,errors,is_binary,find_binary,force_readable=_B,force_writable=_B):
	C=encoding;B=errors;A=text_stream
	if is_binary(A,_B):D=A
	else:
		if _is_compatible_text_stream(A,C,B)and not(C is _A and _stream_is_misconfigured(A)):return A
		D=find_binary(A)
		if D is _A:return A
	if B is _A:B=_D
	return _make_text_stream(D,C,B,force_readable=force_readable,force_writable=force_writable)
def _force_correct_text_reader(text_reader,encoding,errors,force_readable=_B):return _force_correct_text_stream(text_reader,encoding,errors,_is_binary_reader,_find_binary_reader,force_readable=force_readable)
def _force_correct_text_writer(text_writer,encoding,errors,force_writable=_B):return _force_correct_text_stream(text_writer,encoding,errors,_is_binary_writer,_find_binary_writer,force_writable=force_writable)
def get_binary_stdin():
	A=_find_binary_reader(sys.stdin)
	if A is _A:raise RuntimeError('Was not able to determine binary stream for sys.stdin.')
	return A
def get_binary_stdout():
	A=_find_binary_writer(sys.stdout)
	if A is _A:raise RuntimeError('Was not able to determine binary stream for sys.stdout.')
	return A
def get_binary_stderr():
	A=_find_binary_writer(sys.stderr)
	if A is _A:raise RuntimeError('Was not able to determine binary stream for sys.stderr.')
	return A
def get_text_stdin(encoding=_A,errors=_A):
	B=errors;A=encoding;C=_get_windows_console_stream(sys.stdin,A,B)
	if C is not _A:return C
	return _force_correct_text_reader(sys.stdin,A,B,force_readable=_C)
def get_text_stdout(encoding=_A,errors=_A):
	B=errors;A=encoding;C=_get_windows_console_stream(sys.stdout,A,B)
	if C is not _A:return C
	return _force_correct_text_writer(sys.stdout,A,B,force_writable=_C)
def get_text_stderr(encoding=_A,errors=_A):
	B=errors;A=encoding;C=_get_windows_console_stream(sys.stderr,A,B)
	if C is not _A:return C
	return _force_correct_text_writer(sys.stderr,A,B,force_writable=_C)
def filename_to_ui(value):
	A=value
	if isinstance(A,bytes):A=A.decode(get_filesystem_encoding(),_D)
	else:A=A.encode(_F,'surrogateescape').decode(_F,_D)
	return A
def get_strerror(e,default=_A):
	B=default
	if hasattr(e,'strerror'):A=e.strerror
	elif B is not _A:A=B
	else:A=str(e)
	if isinstance(A,bytes):A=A.decode(_F,_D)
	return A
def _wrap_io_open(file,mode,encoding,errors):
	A=mode
	if'b'in A:return open(file,A)
	return open(file,A,encoding=encoding,errors=errors)
def open_stream(filename,mode='r',encoding=_A,errors='strict',atomic=_B):
	P='x';O='a';N='w';E=errors;D=encoding;B=filename;A=mode;G='b'in A
	if B=='-':
		if any((B in A for B in[N,O,P])):
			if G:return get_binary_stdout(),_B
			return get_text_stdout(encoding=D,errors=E),_B
		if G:return get_binary_stdin(),_B
		return get_text_stdin(encoding=D,errors=E),_B
	if not atomic:return _wrap_io_open(B,A,D,E),_C
	if O in A:raise ValueError("Appending to an existing file is not supported, because that would involve an expensive `copy`-operation to a temporary file. Open the file in normal `w`-mode and copy explicitly if that's what you're after.")
	if P in A:raise ValueError('Use the `overwrite`-parameter instead.')
	if N not in A:raise ValueError('Atomic writes only make sense with `w`-mode.')
	import errno as I,random as K
	try:C=os.stat(B).st_mode
	except OSError:C=_A
	J=os.O_RDWR|os.O_CREAT|os.O_EXCL
	if G:J|=getattr(os,'O_BINARY',0)
	while _C:
		H=os.path.join(os.path.dirname(B),f".__atomic-write{K.randrange(1<<32):08x}")
		try:L=os.open(H,J,438 if C is _A else C);break
		except OSError as F:
			if F.errno==I.EEXIST or os.name=='nt'and F.errno==I.EACCES and os.path.isdir(F.filename)and os.access(F.filename,os.W_OK):continue
			raise
	if C is not _A:os.chmod(H,C)
	M=_wrap_io_open(L,A,D,E);return _AtomicFile(M,H,os.path.realpath(B)),_C
class _AtomicFile:
	def __init__(A,f,tmp_filename,real_filename):A._f=f;A._tmp_filename=tmp_filename;A._real_filename=real_filename;A.closed=_B
	@property
	def name(self):return self._real_filename
	def close(A,delete=_B):
		if A.closed:return
		A._f.close();os.replace(A._tmp_filename,A._real_filename);A.closed=_C
	def __getattr__(A,name):return getattr(A._f,name)
	def __enter__(A):return A
	def __exit__(A,exc_type,exc_value,tb):A.close(delete=exc_type is not _A)
	def __repr__(A):return repr(A._f)
def strip_ansi(value):return _ansi_re.sub('',value)
def _is_jupyter_kernel_output(stream):
	A=stream
	if WIN:return
	while isinstance(A,(_FixupStream,_NonClosingTextIOWrapper)):A=A._stream
	return A.__class__.__module__.startswith('ipykernel.')
def should_strip_ansi(stream=_A,color=_A):
	B=color;A=stream
	if B is _A:
		if A is _A:A=sys.stdin
		return not isatty(A)and not _is_jupyter_kernel_output(A)
	return not B
if WIN:
	DEFAULT_COLUMNS=79;from ._winconsole import _get_windows_console_stream
	def _get_argv_encoding():import locale as A;return A.getpreferredencoding()
	try:import colorama
	except ImportError:pass
	else:
		_ansi_stream_wrappers=WeakKeyDictionary()
		def auto_wrap_for_ansi(stream,color=_A):
			A=stream
			try:C=_ansi_stream_wrappers.get(A)
			except Exception:C=_A
			if C is not _A:return C
			E=should_strip_ansi(A,color);D=colorama.AnsiToWin32(A,strip=E);B=D.stream;F=B.write
			def G(s):
				try:return F(s)
				except BaseException:D.reset_all();raise
			B.write=G
			try:_ansi_stream_wrappers[A]=B
			except Exception:pass
			return B
		def get_winterm_size():A=colorama.win32.GetConsoleScreenBufferInfo(colorama.win32.STDOUT).srWindow;return A.Right-A.Left,A.Bottom-A.Top
else:
	def _get_argv_encoding():return getattr(sys.stdin,_E,_A)or get_filesystem_encoding()
	def _get_windows_console_stream(f,encoding,errors):return _A
def term_len(x):return len(strip_ansi(x))
def isatty(stream):
	try:return stream.isatty()
	except Exception:return _B
def _make_cached_stream_func(src_func,wrapper_func):
	C=src_func;D=WeakKeyDictionary()
	def A():
		B=C()
		try:A=D.get(B)
		except Exception:A=_A
		if A is not _A:return A
		A=wrapper_func()
		try:B=C();D[B]=A
		except Exception:pass
		return A
	return A
_default_text_stdin=_make_cached_stream_func(lambda:sys.stdin,get_text_stdin)
_default_text_stdout=_make_cached_stream_func(lambda:sys.stdout,get_text_stdout)
_default_text_stderr=_make_cached_stream_func(lambda:sys.stderr,get_text_stderr)
binary_streams={_J:get_binary_stdin,_K:get_binary_stdout,_L:get_binary_stderr}
text_streams={_J:get_text_stdin,_K:get_text_stdout,_L:get_text_stderr}


from dynaconf.base import LazySettings  # noqa
from dynaconf.constants import DEFAULT_SETTINGS_FILES
from dynaconf.contrib import DjangoDynaconf  # noqa
from dynaconf.contrib import FlaskDynaconf  # noqa
from dynaconf.validator import ValidationError  # noqa
from dynaconf.validator import Validator  # noqa

settings = LazySettings(
    # This global `settings` is deprecated from v3.0.0+
    # kept here for backwards compatibility
    # To Be Removed in 4.0.x
    warn_dynaconf_global_settings=True,
    environments=True,
    lowercase_read=False,
    load_dotenv=True,
    default_settings_paths=DEFAULT_SETTINGS_FILES,
)

# This is the new recommended base class alias
Dynaconf = LazySettings  # noqa

__all__ = [
    "Dynaconf",
    "LazySettings",
    "Validator",
    "FlaskDynaconf",
    "ValidationError",
    "DjangoDynaconf",
]



from __future__ import print_function,absolute_import,division,unicode_literals
_D='comment'
_C='\n'
_B=False
_A=None
from .compat import text_type
from .anchor import Anchor
if _B:from typing import Text,Any,Dict,List
__all__=['ScalarString','LiteralScalarString','FoldedScalarString','SingleQuotedScalarString','DoubleQuotedScalarString','PlainScalarString','PreservedScalarString']
class ScalarString(text_type):
	__slots__=Anchor.attrib
	def __new__(D,*E,**A):
		B=A.pop('anchor',_A);C=text_type.__new__(D,*E,**A)
		if B is not _A:C.yaml_set_anchor(B,always_dump=True)
		return C
	def replace(A,old,new,maxreplace=-1):return type(A)(text_type.replace(A,old,new,maxreplace))
	@property
	def anchor(self):
		A=self
		if not hasattr(A,Anchor.attrib):setattr(A,Anchor.attrib,Anchor())
		return getattr(A,Anchor.attrib)
	def yaml_anchor(A,any=_B):
		if not hasattr(A,Anchor.attrib):return _A
		if any or A.anchor.always_dump:return A.anchor
		return _A
	def yaml_set_anchor(A,value,always_dump=_B):A.anchor.value=value;A.anchor.always_dump=always_dump
class LiteralScalarString(ScalarString):
	__slots__=_D;style='|'
	def __new__(A,value,anchor=_A):return ScalarString.__new__(A,value,anchor=anchor)
PreservedScalarString=LiteralScalarString
class FoldedScalarString(ScalarString):
	__slots__='fold_pos',_D;style='>'
	def __new__(A,value,anchor=_A):return ScalarString.__new__(A,value,anchor=anchor)
class SingleQuotedScalarString(ScalarString):
	__slots__=();style="'"
	def __new__(A,value,anchor=_A):return ScalarString.__new__(A,value,anchor=anchor)
class DoubleQuotedScalarString(ScalarString):
	__slots__=();style='"'
	def __new__(A,value,anchor=_A):return ScalarString.__new__(A,value,anchor=anchor)
class PlainScalarString(ScalarString):
	__slots__=();style=''
	def __new__(A,value,anchor=_A):return ScalarString.__new__(A,value,anchor=anchor)
def preserve_literal(s):return LiteralScalarString(s.replace('\r\n',_C).replace('\r',_C))
def walk_tree(base,map=_A):
	A=base;from dynaconf.vendor.ruamel.yaml.compat import string_types as E,MutableMapping as G,MutableSequence as H
	if map is _A:map={_C:preserve_literal}
	if isinstance(A,G):
		for F in A:
			C=A[F]
			if isinstance(C,E):
				for B in map:
					if B in C:A[F]=map[B](C);break
			else:walk_tree(C)
	elif isinstance(A,H):
		for (I,D) in enumerate(A):
			if isinstance(D,E):
				for B in map:
					if B in D:A[I]=map[B](D);break
			else:walk_tree(D)


_E=True
_D=False
_C=' '
_B='\n'
_A=None
from contextlib import contextmanager
from ._compat import term_len
from .parser import split_opt
from .termui import get_terminal_size
FORCED_WIDTH=_A
def measure_table(rows):
	A={}
	for C in rows:
		for (B,D) in enumerate(C):A[B]=max(A.get(B,0),term_len(D))
	return tuple((B for(C,B)in sorted(A.items())))
def iter_rows(rows,col_count):
	for A in rows:A=tuple(A);yield A+('',)*(col_count-len(A))
def wrap_text(text,width=78,initial_indent='',subsequent_indent='',preserve_paragraphs=_D):
	A=text;from ._textwrap import TextWrapper as I;A=A.expandtabs();E=I(width,initial_indent=initial_indent,subsequent_indent=subsequent_indent,replace_whitespace=_D)
	if not preserve_paragraphs:return E.fill(A)
	F=[];C=[];B=_A
	def H():
		if not C:return
		if C[0].strip()=='\x08':F.append((B or 0,_E,_B.join(C[1:])))
		else:F.append((B or 0,_D,_C.join(C)))
		del C[:]
	for D in A.splitlines():
		if not D:H();B=_A
		else:
			if B is _A:J=term_len(D);D=D.lstrip();B=J-term_len(D)
			C.append(D)
	H();G=[]
	for (B,K,A) in F:
		with E.extra_indent(_C*B):
			if K:G.append(E.indent_only(A))
			else:G.append(E.fill(A))
	return '\n\n'.join(G)
class HelpFormatter:
	def __init__(B,indent_increment=2,width=_A,max_width=_A):
		C=max_width;A=width;B.indent_increment=indent_increment
		if C is _A:C=80
		if A is _A:
			A=FORCED_WIDTH
			if A is _A:A=max(min(get_terminal_size()[0],C)-2,50)
		B.width=A;B.current_indent=0;B.buffer=[]
	def write(A,string):A.buffer.append(string)
	def indent(A):A.current_indent+=A.indent_increment
	def dedent(A):A.current_indent-=A.indent_increment
	def write_usage(A,prog,args='',prefix='Usage: '):
		E=prefix;B=f"{E:>{A.current_indent}}{prog} ";D=A.width-A.current_indent
		if D>=term_len(B)+20:C=_C*term_len(B);A.write(wrap_text(args,D,initial_indent=B,subsequent_indent=C))
		else:A.write(B);A.write(_B);C=_C*(max(A.current_indent,term_len(E))+4);A.write(wrap_text(args,D,initial_indent=C,subsequent_indent=C))
		A.write(_B)
	def write_heading(A,heading):A.write(f"{'':>{A.current_indent}}{heading}:\n")
	def write_paragraph(A):
		if A.buffer:A.write(_B)
	def write_text(A,text):C=max(A.width-A.current_indent,11);B=_C*A.current_indent;A.write(wrap_text(text,C,initial_indent=B,subsequent_indent=B,preserve_paragraphs=_E));A.write(_B)
	def write_dl(A,rows,col_max=30,col_spacing=2):
		G=col_spacing;C=rows;C=list(C);E=measure_table(C)
		if len(E)!=2:raise TypeError('Expected two columns for definition list')
		B=min(E[0],col_max)+G
		for (F,H) in iter_rows(C,len(E)):
			A.write(f"{'':>{A.current_indent}}{F}")
			if not H:A.write(_B);continue
			if term_len(F)<=B-G:A.write(_C*(B-term_len(F)))
			else:A.write(_B);A.write(_C*(B+A.current_indent))
			I=max(A.width-B-2,10);J=wrap_text(H,I,preserve_paragraphs=_E);D=J.splitlines()
			if D:
				A.write(f"{D[0]}\n")
				for K in D[1:]:A.write(f"{'':>{B+A.current_indent}}{K}\n")
				if len(D)>1:A.write(_B)
			else:A.write(_B)
	@contextmanager
	def section(self,name):
		A=self;A.write_paragraph();A.write_heading(name);A.indent()
		try:yield
		finally:A.dedent()
	@contextmanager
	def indentation(self):
		self.indent()
		try:yield
		finally:self.dedent()
	def getvalue(A):return ''.join(A.buffer)
def join_options(options):
	A=[];B=_D
	for C in options:
		D=split_opt(C)[0]
		if D=='/':B=_E
		A.append((len(D),C))
	A.sort(key=lambda x:x[0]);A=', '.join((B[1]for B in A));return A,B


import io
import warnings

from dynaconf.utils import build_env_list
from dynaconf.utils import ensure_a_list
from dynaconf.utils import upperfy


class BaseLoader(object):
    """Base loader for dynaconf source files.

    :param obj: {[LazySettings]} -- [Dynaconf settings]
    :param env: {[string]} -- [the current env to be loaded defaults to
      [development]]
    :param identifier: {[string]} -- [identifier ini, yaml, json, py, toml]
    :param extensions: {[list]} -- [List of extensions with dots ['.a', '.b']]
    :param file_reader: {[callable]} -- [reads file return dict]
    :param string_reader: {[callable]} -- [reads string return dict]
    """

    def __init__(
        self, obj, env, identifier, extensions, file_reader, string_reader
    ):
        """Instantiates a loader for different sources"""
        self.obj = obj
        self.env = env or obj.current_env
        self.identifier = identifier
        self.extensions = extensions
        self.file_reader = file_reader
        self.string_reader = string_reader

    @staticmethod
    def warn_not_installed(obj, identifier):  # pragma: no cover
        if identifier not in obj._not_installed_warnings:
            warnings.warn(
                f"{identifier} support is not installed in your environment. "
                f"`pip install dynaconf[{identifier}]`"
            )
        obj._not_installed_warnings.append(identifier)

    def load(self, filename=None, key=None, silent=True):
        """
        Reads and loads in to `self.obj` a single key or all keys from source

        :param filename: Optional filename to load
        :param key: if provided load a single key
        :param silent: if load erros should be silenced
        """
        filename = filename or self.obj.get(self.identifier.upper())
        if not filename:
            return

        if not isinstance(filename, (list, tuple)):
            split_files = ensure_a_list(filename)
            if all([f.endswith(self.extensions) for f in split_files]):  # noqa
                files = split_files  # it is a ['file.ext', ...]
            else:  # it is a single config as string
                files = [filename]
        else:  # it is already a list/tuple
            files = filename

        source_data = self.get_source_date(files)

        if self.obj.get("ENVIRONMENTS_FOR_DYNACONF") is False:
            self._envless_load(source_data, silent, key)
        else:
            self._load_all_envs(source_data, silent, key)

    def get_source_date(self, files):
        """Reads each file and returns source data for each file
        {"path/to/file.ext": {"key": "value"}}
        """
        data = {}
        for source_file in files:
            if source_file.endswith(self.extensions):
                try:
                    with io.open(
                        source_file,
                        encoding=self.obj.get(
                            "ENCODING_FOR_DYNACONF", "utf-8"
                        ),
                    ) as open_file:
                        content = self.file_reader(open_file)
                        self.obj._loaded_files.append(source_file)
                        if content:
                            data[source_file] = content
                except IOError as e:
                    if ".local." not in source_file:
                        warnings.warn(
                            f"{self.identifier}_loader: {source_file} "
                            f":{str(e)}"
                        )
            else:
                # for tests it is possible to pass string
                content = self.string_reader(source_file)
                if content:
                    data[source_file] = content
        return data

    def _envless_load(self, source_data, silent=True, key=None):
        """Load all the keys from each file without env separation"""
        for source_file, file_data in source_data.items():
            self._set_data_to_obj(
                file_data, self.identifier, source_file, key=key
            )

    def _load_all_envs(self, source_data, silent=True, key=None):
        """Load configs from files separating by each environment"""

        for source_file, file_data in source_data.items():

            # env name is checked in lower
            file_data = {k.lower(): value for k, value in file_data.items()}

            # is there a `dynaconf_merge` on top level of file?
            file_merge = file_data.get("dynaconf_merge")

            # all lower case for comparison
            base_envs = [
                # DYNACONF or MYPROGRAM
                (self.obj.get("ENVVAR_PREFIX_FOR_DYNACONF") or "").lower(),
                # DEFAULT
                self.obj.get("DEFAULT_ENV_FOR_DYNACONF").lower(),
                # default active env unless ENV_FOR_DYNACONF is changed
                "development",
                # backwards compatibility for global
                "dynaconf",
                # global that rules all
                "global",
            ]

            for env in build_env_list(self.obj, self.env):
                env = env.lower()  # lower for better comparison
                data = {}
                try:
                    data = file_data[env] or {}
                except KeyError:
                    if env not in base_envs:
                        message = (
                            f"{self.identifier}_loader: {env} env not"
                            f"defined in {source_file}"
                        )
                        if silent:
                            warnings.warn(message)
                        else:
                            raise KeyError(message)
                    continue

                if env != self.obj.get("DEFAULT_ENV_FOR_DYNACONF").lower():
                    identifier = f"{self.identifier}_{env}"
                else:
                    identifier = self.identifier

                self._set_data_to_obj(
                    data, identifier, source_file, file_merge, key, env
                )

    def _set_data_to_obj(
        self,
        data,
        identifier,
        source_file,
        file_merge=None,
        key=False,
        env=False,
    ):
        """Calls setttings.set to add the keys"""

        # data 1st level keys should be transformed to upper case.
        data = {upperfy(k): v for k, v in data.items()}
        if key:
            key = upperfy(key)

        is_secret = "secret" in source_file

        # is there a `dynaconf_merge` inside an `[env]`?
        file_merge = file_merge or data.pop("DYNACONF_MERGE", False)

        if not key:
            self.obj.update(
                data,
                loader_identifier=identifier,
                is_secret=is_secret,
                merge=file_merge,
            )
        elif key in data:
            self.obj.set(
                key,
                data.get(key),
                loader_identifier=identifier,
                is_secret=is_secret,
                merge=file_merge,
            )



_H='replace'
_G='less'
_F='You need to use progress bars in a with block.'
_E=' '
_D='\n'
_C=False
_B=True
_A=None
import contextlib,math,os,sys,time
from ._compat import _default_text_stdout,CYGWIN,get_best_encoding,isatty,open_stream,strip_ansi,term_len,WIN
from .exceptions import ClickException
from .utils import echo
if os.name=='nt':BEFORE_BAR='\r';AFTER_BAR=_D
else:BEFORE_BAR='\r\x1b[?25l';AFTER_BAR='\x1b[?25h\n'
def _length_hint(obj):
	B=obj
	try:return len(B)
	except (AttributeError,TypeError):
		try:C=type(B).__length_hint__
		except AttributeError:return _A
		try:A=C(B)
		except TypeError:return _A
		if A is NotImplemented or not isinstance(A,int)or A<0:return _A
		return A
class ProgressBar:
	def __init__(A,iterable,length=_A,fill_char='#',empty_char=_E,bar_template='%(bar)s',info_sep='  ',show_eta=_B,show_percent=_A,show_pos=_C,item_show_func=_A,label=_A,file=_A,color=_A,width=30):
		E=width;D=file;C=iterable;B=length;A.fill_char=fill_char;A.empty_char=empty_char;A.bar_template=bar_template;A.info_sep=info_sep;A.show_eta=show_eta;A.show_percent=show_percent;A.show_pos=show_pos;A.item_show_func=item_show_func;A.label=label or''
		if D is _A:D=_default_text_stdout()
		A.file=D;A.color=color;A.width=E;A.autowidth=E==0
		if B is _A:B=_length_hint(C)
		if C is _A:
			if B is _A:raise TypeError('iterable or length is required')
			C=range(B)
		A.iter=iter(C);A.length=B;A.length_known=B is not _A;A.pos=0;A.avg=[];A.start=A.last_eta=time.time();A.eta_known=_C;A.finished=_C;A.max_width=_A;A.entered=_C;A.current_item=_A;A.is_hidden=not isatty(A.file);A._last_line=_A;A.short_limit=0.5
	def __enter__(A):A.entered=_B;A.render_progress();return A
	def __exit__(A,exc_type,exc_value,tb):A.render_finish()
	def __iter__(A):
		if not A.entered:raise RuntimeError(_F)
		A.render_progress();return A.generator()
	def __next__(A):return next(iter(A))
	def is_fast(A):return time.time()-A.start<=A.short_limit
	def render_finish(A):
		if A.is_hidden or A.is_fast():return
		A.file.write(AFTER_BAR);A.file.flush()
	@property
	def pct(self):
		A=self
		if A.finished:return 1.0
		return min(A.pos/(float(A.length)or 1),1.0)
	@property
	def time_per_iteration(self):
		A=self
		if not A.avg:return 0.0
		return sum(A.avg)/float(len(A.avg))
	@property
	def eta(self):
		A=self
		if A.length_known and not A.finished:return A.time_per_iteration*(A.length-A.pos)
		return 0.0
	def format_eta(B):
		if B.eta_known:
			A=int(B.eta);C=A%60;A//=60;D=A%60;A//=60;E=A%24;A//=24
			if A>0:return f"{A}d {E:02}:{D:02}:{C:02}"
			else:return f"{E:02}:{D:02}:{C:02}"
		return''
	def format_pos(A):
		B=str(A.pos)
		if A.length_known:B+=f"/{A.length}"
		return B
	def format_pct(A):return f"{int(A.pct*100): 4}%"[1:]
	def format_bar(A):
		if A.length_known:C=int(A.pct*A.width);B=A.fill_char*C;B+=A.empty_char*(A.width-C)
		elif A.finished:B=A.fill_char*A.width
		else:
			B=list(A.empty_char*(A.width or 1))
			if A.time_per_iteration!=0:B[int((math.cos(A.pos*A.time_per_iteration)/2.0+0.5)*A.width)]=A.fill_char
			B=''.join(B)
		return B
	def format_progress_line(A):
		C=A.show_percent;B=[]
		if A.length_known and C is _A:C=not A.show_pos
		if A.show_pos:B.append(A.format_pos())
		if C:B.append(A.format_pct())
		if A.show_eta and A.eta_known and not A.finished:B.append(A.format_eta())
		if A.item_show_func is not _A:
			D=A.item_show_func(A.current_item)
			if D is not _A:B.append(D)
		return (A.bar_template%{'label':A.label,'bar':A.format_bar(),'info':A.info_sep.join(B)}).rstrip()
	def render_progress(A):
		from .termui import get_terminal_size as G
		if A.is_hidden:return
		B=[]
		if A.autowidth:
			H=A.width;A.width=0;I=term_len(A.format_progress_line());D=max(0,G()[0]-I)
			if D<H:B.append(BEFORE_BAR);B.append(_E*A.max_width);A.max_width=D
			A.width=D
		F=A.width
		if A.max_width is not _A:F=A.max_width
		B.append(BEFORE_BAR);C=A.format_progress_line();E=term_len(C)
		if A.max_width is _A or A.max_width<E:A.max_width=E
		B.append(C);B.append(_E*(F-E));C=''.join(B)
		if C!=A._last_line and not A.is_fast():A._last_line=C;echo(C,file=A.file,color=A.color,nl=_C);A.file.flush()
	def make_step(A,n_steps):
		A.pos+=n_steps
		if A.length_known and A.pos>=A.length:A.finished=_B
		if time.time()-A.last_eta<1.0:return
		A.last_eta=time.time()
		if A.pos:B=(time.time()-A.start)/A.pos
		else:B=time.time()-A.start
		A.avg=A.avg[-6:]+[B];A.eta_known=A.length_known
	def update(A,n_steps,current_item=_A):
		B=current_item;A.make_step(n_steps)
		if B is not _A:A.current_item=B
		A.render_progress()
	def finish(A):A.eta_known=0;A.current_item=_A;A.finished=_B
	def generator(A):
		if not A.entered:raise RuntimeError(_F)
		if A.is_hidden:yield from A.iter
		else:
			for B in A.iter:A.current_item=B;yield B;A.update(1)
			A.finish();A.render_progress()
def pager(generator,color=_A):
	H='system';B=color;A=generator;C=_default_text_stdout()
	if not isatty(sys.stdin)or not isatty(C):return _nullpager(C,A,B)
	D=(os.environ.get('PAGER',_A)or'').strip()
	if D:
		if WIN:return _tempfilepager(A,D,B)
		return _pipepager(A,D,B)
	if os.environ.get('TERM')in('dumb','emacs'):return _nullpager(C,A,B)
	if WIN or sys.platform.startswith('os2'):return _tempfilepager(A,'more <',B)
	if hasattr(os,H)and os.system('(less) 2>/dev/null')==0:return _pipepager(A,_G,B)
	import tempfile as F;G,E=F.mkstemp();os.close(G)
	try:
		if hasattr(os,H)and os.system(f'more "{E}"')==0:return _pipepager(A,'more',B)
		return _nullpager(C,A,B)
	finally:os.unlink(E)
def _pipepager(generator,cmd,color):
	I='LESS';A=color;import subprocess as E;F=dict(os.environ);G=cmd.rsplit('/',1)[-1].split()
	if A is _A and G[0]==_G:
		C=f"{os.environ.get(I,'')}{_E.join(G[1:])}"
		if not C:F[I]='-R';A=_B
		elif'r'in C or'R'in C:A=_B
	B=E.Popen(cmd,shell=_B,stdin=E.PIPE,env=F);H=get_best_encoding(B.stdin)
	try:
		for D in generator:
			if not A:D=strip_ansi(D)
			B.stdin.write(D.encode(H,_H))
	except (OSError,KeyboardInterrupt):pass
	else:B.stdin.close()
	while _B:
		try:B.wait()
		except KeyboardInterrupt:pass
		else:break
def _tempfilepager(generator,cmd,color):
	import tempfile as C;A=C.mktemp();B=''.join(generator)
	if not color:B=strip_ansi(B)
	D=get_best_encoding(sys.stdout)
	with open_stream(A,'wb')[0]as E:E.write(B.encode(D))
	try:os.system(f'{cmd} "{A}"')
	finally:os.unlink(A)
def _nullpager(stream,generator,color):
	for A in generator:
		if not color:A=strip_ansi(A)
		stream.write(A)
class Editor:
	def __init__(A,editor=_A,env=_A,require_save=_B,extension='.txt'):A.editor=editor;A.env=env;A.require_save=require_save;A.extension=extension
	def get_editor(A):
		if A.editor is not _A:return A.editor
		for D in ('VISUAL','EDITOR'):
			B=os.environ.get(D)
			if B:return B
		if WIN:return'notepad'
		for C in ('sensible-editor','vim','nano'):
			if os.system(f"which {C} >/dev/null 2>&1")==0:return C
		return'vi'
	def edit_file(A,filename):
		import subprocess as D;B=A.get_editor()
		if A.env:C=os.environ.copy();C.update(A.env)
		else:C=_A
		try:
			E=D.Popen(f'{B} "{filename}"',env=C,shell=_B);F=E.wait()
			if F!=0:raise ClickException(f"{B}: Editing failed!")
		except OSError as G:raise ClickException(f"{B}: Editing failed: {G}")
	def edit(D,text):
		L='\r\n';K='utf-8-sig';A=text;import tempfile as H;A=A or'';E=type(A)in[bytes,bytearray]
		if not E and A and not A.endswith(_D):A+=_D
		I,B=H.mkstemp(prefix='editor-',suffix=D.extension)
		try:
			if not E:
				if WIN:F=K;A=A.replace(_D,L)
				else:F='utf-8'
				A=A.encode(F)
			C=os.fdopen(I,'wb');C.write(A);C.close();J=os.path.getmtime(B);D.edit_file(B)
			if D.require_save and os.path.getmtime(B)==J:return _A
			C=open(B,'rb')
			try:G=C.read()
			finally:C.close()
			if E:return G
			else:return G.decode(K).replace(L,_D)
		finally:os.unlink(B)
def open_url(url,wait=_C,locate=_C):
	F='"';D=locate;C=wait;A=url;import subprocess as G
	def E(url):
		A=url;import urllib as B
		if A.startswith('file://'):A=B.unquote(A[7:])
		return A
	if sys.platform=='darwin':
		B=['open']
		if C:B.append('-W')
		if D:B.append('-R')
		B.append(E(A));H=open('/dev/null','w')
		try:return G.Popen(B,stderr=H).wait()
		finally:H.close()
	elif WIN:
		if D:A=E(A.replace(F,''));B=f'explorer /select,"{A}"'
		else:A=A.replace(F,'');C='/WAIT'if C else'';B=f'start {C} "" "{A}"'
		return os.system(B)
	elif CYGWIN:
		if D:A=os.path.dirname(E(A).replace(F,''));B=f'cygstart "{A}"'
		else:A=A.replace(F,'');C='-w'if C else'';B=f'cygstart {C} "{A}"'
		return os.system(B)
	try:
		if D:A=os.path.dirname(E(A))or'.'
		else:A=E(A)
		I=G.Popen(['xdg-open',A])
		if C:return I.wait()
		return 0
	except OSError:
		if A.startswith(('http://','https://'))and not D and not C:import webbrowser as J;J.open(A);return 0
		return 1
def _translate_ch_to_exc(ch):
	if ch=='\x03':raise KeyboardInterrupt()
	if ch=='\x04'and not WIN:raise EOFError()
	if ch=='\x1a'and WIN:raise EOFError()
if WIN:
	import msvcrt
	@contextlib.contextmanager
	def raw_terminal():yield
	def getchar(echo):
		if echo:B=msvcrt.getwche
		else:B=msvcrt.getwch
		A=B()
		if A in('\x00','à'):A+=B()
		_translate_ch_to_exc(A);return A
else:
	import tty,termios
	@contextlib.contextmanager
	def raw_terminal():
		if not isatty(sys.stdin):B=open('/dev/tty');A=B.fileno()
		else:A=sys.stdin.fileno();B=_A
		try:
			C=termios.tcgetattr(A)
			try:tty.setraw(A);yield A
			finally:
				termios.tcsetattr(A,termios.TCSADRAIN,C);sys.stdout.flush()
				if B is not _A:B.close()
		except termios.error:pass
	def getchar(echo):
		with raw_terminal()as B:
			A=os.read(B,32);A=A.decode(get_best_encoding(sys.stdin),_H)
			if echo and isatty(sys.stdout):sys.stdout.write(A)
			_translate_ch_to_exc(A);return A


_A='utf-8'
import sys
PY2=sys.version_info[0]==2
if PY2:from StringIO import StringIO
else:from io import StringIO
def is_type_checking():
	try:from typing import TYPE_CHECKING as A
	except ImportError:return False
	return A
IS_TYPE_CHECKING=is_type_checking()
if IS_TYPE_CHECKING:from typing import Text
def to_env(text):
	if PY2:return text.encode(sys.getfilesystemencoding()or _A)
	else:return text
def to_text(string):
	A=string
	if PY2:return A.decode(_A)
	else:return A


import inspect
import io
import os

from dynaconf.utils import deduplicate


def _walk_to_root(path, break_at=None):
    """
    Directories starting from the given directory up to the root or break_at
    """
    if not os.path.exists(path):  # pragma: no cover
        raise IOError("Starting path not found")

    if os.path.isfile(path):  # pragma: no cover
        path = os.path.dirname(path)

    last_dir = None
    current_dir = os.path.abspath(path)
    paths = []
    while last_dir != current_dir:
        paths.append(current_dir)
        paths.append(os.path.join(current_dir, "config"))
        if break_at and current_dir == os.path.abspath(break_at):  # noqa
            break
        parent_dir = os.path.abspath(os.path.join(current_dir, os.path.pardir))
        last_dir, current_dir = current_dir, parent_dir
    return paths


SEARCHTREE = []


def find_file(filename=".env", project_root=None, skip_files=None, **kwargs):
    """Search in increasingly higher folders for the given file
    Returns path to the file if found, or an empty string otherwise.

    This function will build a `search_tree` based on:

    - Project_root if specified
    - Invoked script location and its parents until root
    - Current working directory

    For each path in the `search_tree` it will also look for an
    aditional `./config` folder.
    """
    search_tree = []
    work_dir = os.getcwd()
    skip_files = skip_files or []

    if project_root is not None:
        search_tree.extend(_walk_to_root(project_root, break_at=work_dir))

    script_dir = os.path.dirname(os.path.abspath(inspect.stack()[-1].filename))

    # Path to invoked script and recursively to root with its ./config dirs
    search_tree.extend(_walk_to_root(script_dir))

    # Path to where Python interpreter was invoked and recursively to root
    search_tree.extend(_walk_to_root(work_dir))

    # Don't look the same place twice
    search_tree = deduplicate(search_tree)

    global SEARCHTREE
    SEARCHTREE[:] = search_tree

    for dirname in search_tree:
        check_path = os.path.join(dirname, filename)
        if check_path in skip_files:
            continue
        if os.path.exists(check_path):
            return check_path  # First found will return

    # return empty string if not found so it can still be joined in os.path
    return ""


def read_file(path, **kwargs):
    content = ""
    with io.open(path, **kwargs) as open_file:
        content = open_file.read().strip()
    return content


def get_local_filename(filename):
    """Takes a filename like `settings.toml` and returns `settings.local.toml`

    Arguments:
        filename {str} -- The filename or complete path

    Returns:
        [str] -- The same name or path with `.local.` added.
    """
    name, _, extension = os.path.basename(str(filename)).rpartition(
        os.path.extsep
    )

    return os.path.join(
        os.path.dirname(str(filename)), f"{name}.local.{extension}"
    )



import importlib
import os
import sys
import warnings

from dynaconf.utils import RENAMED_VARS
from dynaconf.utils import upperfy
from dynaconf.utils import warn_deprecations
from dynaconf.utils.files import find_file
from dynaconf.utils.parse_conf import parse_conf_data
from dynaconf.vendor.dotenv import load_dotenv


def try_renamed(key, value, older_key, current_key):
    if value is None:
        if key == current_key:
            if older_key in os.environ:
                warnings.warn(
                    f"{older_key} is deprecated please use {current_key}",
                    DeprecationWarning,
                )
                value = os.environ[older_key]
    return value


def get(key, default=None):
    value = os.environ.get(upperfy(key))

    # compatibility with renamed variables
    for old, new in RENAMED_VARS.items():
        value = try_renamed(key, value, old, new)

    return (
        parse_conf_data(value, tomlfy=True, box_settings={})
        if value is not None
        else default
    )


def start_dotenv(obj=None, root_path=None):
    # load_from_dotenv_if_installed
    obj = obj or {}
    _find_file = getattr(obj, "find_file", find_file)
    root_path = (
        root_path
        or getattr(obj, "_root_path", None)
        or get("ROOT_PATH_FOR_DYNACONF")
    )

    dotenv_path = (
        obj.get("DOTENV_PATH_FOR_DYNACONF")
        or get("DOTENV_PATH_FOR_DYNACONF")
        or _find_file(".env", project_root=root_path)
    )

    load_dotenv(
        dotenv_path,
        verbose=obj.get("DOTENV_VERBOSE_FOR_DYNACONF", False),
        override=obj.get("DOTENV_OVERRIDE_FOR_DYNACONF", False),
    )

    warn_deprecations(os.environ)


def reload(load_dotenv=None, *args, **kwargs):
    if load_dotenv:
        start_dotenv(*args, **kwargs)
    importlib.reload(sys.modules[__name__])


# default proj root
# pragma: no cover
ROOT_PATH_FOR_DYNACONF = get("ROOT_PATH_FOR_DYNACONF", None)

# Default settings file
SETTINGS_FILE_FOR_DYNACONF = get("SETTINGS_FILE_FOR_DYNACONF", [])

# MISPELLS `FILES` when/if it happens
mispelled_files = get("SETTINGS_FILES_FOR_DYNACONF", None)
if not SETTINGS_FILE_FOR_DYNACONF and mispelled_files is not None:
    SETTINGS_FILE_FOR_DYNACONF = mispelled_files

# # ENV SETTINGS
# # In dynaconf 1.0.0 `NAMESPACE` got renamed to `ENV`


# If provided environments will be loaded separatelly
ENVIRONMENTS_FOR_DYNACONF = get("ENVIRONMENTS_FOR_DYNACONF", False)

# If False dynaconf will allow access to first level settings only in upper
LOWERCASE_READ_FOR_DYNACONF = get("LOWERCASE_READ_FOR_DYNACONF", True)

# The environment variable to switch current env
ENV_SWITCHER_FOR_DYNACONF = get(
    "ENV_SWITCHER_FOR_DYNACONF", "ENV_FOR_DYNACONF"
)

# The current env by default is DEVELOPMENT
# to switch is needed to `export ENV_FOR_DYNACONF=PRODUCTION`
# or put that value in .env file
# this value is used only when reading files like .toml|yaml|ini|json
ENV_FOR_DYNACONF = get(ENV_SWITCHER_FOR_DYNACONF, "DEVELOPMENT")

# This variable exists to support `from_env` method
FORCE_ENV_FOR_DYNACONF = get("FORCE_ENV_FOR_DYNACONF", None)

# Default values is taken from DEFAULT pseudo env
# this value is used only when reading files like .toml|yaml|ini|json
DEFAULT_ENV_FOR_DYNACONF = get("DEFAULT_ENV_FOR_DYNACONF", "DEFAULT")

# Global values are taken from DYNACONF env used for exported envvars
# Values here overwrites all other envs
# This namespace is used for files and also envvars
ENVVAR_PREFIX_FOR_DYNACONF = get("ENVVAR_PREFIX_FOR_DYNACONF", "DYNACONF")

# The default encoding to open settings files
ENCODING_FOR_DYNACONF = get("ENCODING_FOR_DYNACONF", "utf-8")

# Merge objects on load
MERGE_ENABLED_FOR_DYNACONF = get("MERGE_ENABLED_FOR_DYNACONF", False)

# BY default `__` is the separator for nested env vars
# export `DYNACONF__DATABASE__server=server.com`
# export `DYNACONF__DATABASE__PORT=6666`
# Should result in settings.DATABASE == {'server': 'server.com', 'PORT': 6666}
# To disable it one can set `NESTED_SEPARATOR_FOR_DYNACONF=false`
NESTED_SEPARATOR_FOR_DYNACONF = get("NESTED_SEPARATOR_FOR_DYNACONF", "__")

# The env var specifying settings module
ENVVAR_FOR_DYNACONF = get("ENVVAR_FOR_DYNACONF", "SETTINGS_FILE_FOR_DYNACONF")

# Default values for redis configs
default_redis = {
    "host": get("REDIS_HOST_FOR_DYNACONF", "localhost"),
    "port": int(get("REDIS_PORT_FOR_DYNACONF", 6379)),
    "db": int(get("REDIS_DB_FOR_DYNACONF", 0)),
    "decode_responses": get("REDIS_DECODE_FOR_DYNACONF", True),
    "username": get("REDIS_USERNAME_FOR_DYNACONF", None),
    "password": get("REDIS_PASSWORD_FOR_DYNACONF", None),
}
REDIS_FOR_DYNACONF = get("REDIS_FOR_DYNACONF", default_redis)
REDIS_ENABLED_FOR_DYNACONF = get("REDIS_ENABLED_FOR_DYNACONF", False)

# Hashicorp Vault Project
vault_scheme = get("VAULT_SCHEME_FOR_DYNACONF", "http")
vault_host = get("VAULT_HOST_FOR_DYNACONF", "localhost")
vault_port = get("VAULT_PORT_FOR_DYNACONF", "8200")
default_vault = {
    "url": get(
        "VAULT_URL_FOR_DYNACONF", f"{vault_scheme}://{vault_host}:{vault_port}"
    ),
    "token": get("VAULT_TOKEN_FOR_DYNACONF", None),
    "cert": get("VAULT_CERT_FOR_DYNACONF", None),
    "verify": get("VAULT_VERIFY_FOR_DYNACONF", None),
    "timeout": get("VAULT_TIMEOUT_FOR_DYNACONF", None),
    "proxies": get("VAULT_PROXIES_FOR_DYNACONF", None),
    "allow_redirects": get("VAULT_ALLOW_REDIRECTS_FOR_DYNACONF", None),
}
VAULT_FOR_DYNACONF = get("VAULT_FOR_DYNACONF", default_vault)
VAULT_ENABLED_FOR_DYNACONF = get("VAULT_ENABLED_FOR_DYNACONF", False)
VAULT_PATH_FOR_DYNACONF = get("VAULT_PATH_FOR_DYNACONF", "dynaconf")
VAULT_MOUNT_POINT_FOR_DYNACONF = get(
    "VAULT_MOUNT_POINT_FOR_DYNACONF", "secret"
)
VAULT_ROOT_TOKEN_FOR_DYNACONF = get("VAULT_ROOT_TOKEN_FOR_DYNACONF", None)
VAULT_KV_VERSION_FOR_DYNACONF = get("VAULT_KV_VERSION_FOR_DYNACONF", 1)
VAULT_AUTH_WITH_IAM_FOR_DYNACONF = get(
    "VAULT_AUTH_WITH_IAM_FOR_DYNACONF", False
)
VAULT_AUTH_ROLE_FOR_DYNACONF = get("VAULT_AUTH_ROLE_FOR_DYNACONF", None)
VAULT_ROLE_ID_FOR_DYNACONF = get("VAULT_ROLE_ID_FOR_DYNACONF", None)
VAULT_SECRET_ID_FOR_DYNACONF = get("VAULT_SECRET_ID_FOR_DYNACONF", None)

# Only core loaders defined on this list will be invoked
core_loaders = ["YAML", "TOML", "INI", "JSON", "PY"]
CORE_LOADERS_FOR_DYNACONF = get("CORE_LOADERS_FOR_DYNACONF", core_loaders)

# External Loaders to read vars from different data stores
default_loaders = [
    "dynaconf.loaders.env_loader",
    # 'dynaconf.loaders.redis_loader'
    # 'dynaconf.loaders.vault_loader'
]
LOADERS_FOR_DYNACONF = get("LOADERS_FOR_DYNACONF", default_loaders)

# Errors in loaders should be silenced?
SILENT_ERRORS_FOR_DYNACONF = get("SILENT_ERRORS_FOR_DYNACONF", True)

# always fresh variables
FRESH_VARS_FOR_DYNACONF = get("FRESH_VARS_FOR_DYNACONF", [])

DOTENV_PATH_FOR_DYNACONF = get("DOTENV_PATH_FOR_DYNACONF", None)
DOTENV_VERBOSE_FOR_DYNACONF = get("DOTENV_VERBOSE_FOR_DYNACONF", False)
DOTENV_OVERRIDE_FOR_DYNACONF = get("DOTENV_OVERRIDE_FOR_DYNACONF", False)

# Currently this is only used by cli. INSTANCE_FOR_DYNACONF specifies python
# dotted path to custom LazySettings instance. Last dotted path item should be
# instance of LazySettings.
INSTANCE_FOR_DYNACONF = get("INSTANCE_FOR_DYNACONF", None)

# https://msg.pyyaml.org/load
YAML_LOADER_FOR_DYNACONF = get("YAML_LOADER_FOR_DYNACONF", "safe_load")

# Use commentjson? https://commentjson.readthedocs.io/en/latest/
COMMENTJSON_ENABLED_FOR_DYNACONF = get(
    "COMMENTJSON_ENABLED_FOR_DYNACONF", False
)

# Extra file, or list of files where to look for secrets
# useful for CI environment like jenkins
# where you can export this variable pointing to a local
# absolute path of the secrets file.
SECRETS_FOR_DYNACONF = get("SECRETS_FOR_DYNACONF", None)

# To include extra paths based on envvar
INCLUDES_FOR_DYNACONF = get("INCLUDES_FOR_DYNACONF", [])

# To pre-load extra paths based on envvar
PRELOAD_FOR_DYNACONF = get("PRELOAD_FOR_DYNACONF", [])

# Files to skip if found on search tree
SKIP_FILES_FOR_DYNACONF = get("SKIP_FILES_FOR_DYNACONF", [])


# Backwards compatibility with renamed variables
for old, new in RENAMED_VARS.items():
    setattr(sys.modules[__name__], old, locals()[new])



from __future__ import absolute_import
_A=None
from .emitter import Emitter
from .serializer import Serializer
from .representer import Representer,SafeRepresenter,BaseRepresenter,RoundTripRepresenter
from .resolver import Resolver,BaseResolver,VersionedResolver
if False:from typing import Any,Dict,List,Union,Optional;from .compat import StreamType,VersionType
__all__=['BaseDumper','SafeDumper','Dumper','RoundTripDumper']
class BaseDumper(Emitter,Serializer,BaseRepresenter,BaseResolver):
	def __init__(A,stream,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=_A,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):Emitter.__init__(A,stream,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break,block_seq_indent=block_seq_indent,dumper=A);Serializer.__init__(A,encoding=encoding,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags,dumper=A);BaseRepresenter.__init__(A,default_style=default_style,default_flow_style=default_flow_style,dumper=A);BaseResolver.__init__(A,loadumper=A)
class SafeDumper(Emitter,Serializer,SafeRepresenter,Resolver):
	def __init__(A,stream,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=_A,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):Emitter.__init__(A,stream,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break,block_seq_indent=block_seq_indent,dumper=A);Serializer.__init__(A,encoding=encoding,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags,dumper=A);SafeRepresenter.__init__(A,default_style=default_style,default_flow_style=default_flow_style,dumper=A);Resolver.__init__(A,loadumper=A)
class Dumper(Emitter,Serializer,Representer,Resolver):
	def __init__(A,stream,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=_A,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):Emitter.__init__(A,stream,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break,block_seq_indent=block_seq_indent,dumper=A);Serializer.__init__(A,encoding=encoding,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags,dumper=A);Representer.__init__(A,default_style=default_style,default_flow_style=default_flow_style,dumper=A);Resolver.__init__(A,loadumper=A)
class RoundTripDumper(Emitter,Serializer,RoundTripRepresenter,VersionedResolver):
	def __init__(A,stream,default_style=_A,default_flow_style=_A,canonical=_A,indent=_A,width=_A,allow_unicode=_A,line_break=_A,encoding=_A,explicit_start=_A,explicit_end=_A,version=_A,tags=_A,block_seq_indent=_A,top_level_colon_align=_A,prefix_colon=_A):Emitter.__init__(A,stream,canonical=canonical,indent=indent,width=width,allow_unicode=allow_unicode,line_break=line_break,block_seq_indent=block_seq_indent,top_level_colon_align=top_level_colon_align,prefix_colon=prefix_colon,dumper=A);Serializer.__init__(A,encoding=encoding,explicit_start=explicit_start,explicit_end=explicit_end,version=version,tags=tags,dumper=A);RoundTripRepresenter.__init__(A,default_style=default_style,default_flow_style=default_flow_style,dumper=A);VersionedResolver.__init__(A,loader=A)


_A=None
from ._compat import filename_to_ui,get_text_stderr
from .utils import echo
def _join_param_hints(param_hint):
	A=param_hint
	if isinstance(A,(tuple,list)):return ' / '.join((repr(B)for B in A))
	return A
class ClickException(Exception):
	exit_code=1
	def __init__(B,message):A=message;super().__init__(A);B.message=A
	def format_message(A):return A.message
	def __str__(A):return A.message
	def show(B,file=_A):
		A=file
		if A is _A:A=get_text_stderr()
		echo(f"Error: {B.format_message()}",file=A)
class UsageError(ClickException):
	exit_code=2
	def __init__(A,message,ctx=_A):ClickException.__init__(A,message);A.ctx=ctx;A.cmd=A.ctx.command if A.ctx else _A
	def show(A,file=_A):
		B=file
		if B is _A:B=get_text_stderr()
		C=_A;D=''
		if A.cmd is not _A and A.cmd.get_help_option(A.ctx)is not _A:D=f"Try '{A.ctx.command_path} {A.ctx.help_option_names[0]}' for help.\n"
		if A.ctx is not _A:C=A.ctx.color;echo(f"{A.ctx.get_usage()}\n{D}",file=B,color=C)
		echo(f"Error: {A.format_message()}",file=B,color=C)
class BadParameter(UsageError):
	def __init__(A,message,ctx=_A,param=_A,param_hint=_A):UsageError.__init__(A,message,ctx);A.param=param;A.param_hint=param_hint
	def format_message(A):
		if A.param_hint is not _A:B=A.param_hint
		elif A.param is not _A:B=A.param.get_error_hint(A.ctx)
		else:return f"Invalid value: {A.message}"
		B=_join_param_hints(B);return f"Invalid value for {B}: {A.message}"
class MissingParameter(BadParameter):
	def __init__(A,message=_A,ctx=_A,param=_A,param_hint=_A,param_type=_A):BadParameter.__init__(A,message,ctx,param,param_hint);A.param_type=param_type
	def format_message(A):
		if A.param_hint is not _A:B=A.param_hint
		elif A.param is not _A:B=A.param.get_error_hint(A.ctx)
		else:B=_A
		B=_join_param_hints(B);D=A.param_type
		if D is _A and A.param is not _A:D=A.param.param_type_name
		C=A.message
		if A.param is not _A:
			E=A.param.type.get_missing_message(A.param)
			if E:
				if C:C+=f".  {E}"
				else:C=E
		F=f" {B}"if B else'';return f"Missing {D}{F}.{' 'if C else''}{C or''}"
	def __str__(A):
		if A.message is _A:B=A.param.name if A.param else _A;return f"missing parameter: {B}"
		else:return A.message
class NoSuchOption(UsageError):
	def __init__(A,option_name,message=_A,possibilities=_A,ctx=_A):
		C=option_name;B=message
		if B is _A:B=f"no such option: {C}"
		UsageError.__init__(A,B,ctx);A.option_name=C;A.possibilities=possibilities
	def format_message(A):
		B=[A.message]
		if A.possibilities:
			if len(A.possibilities)==1:B.append(f"Did you mean {A.possibilities[0]}?")
			else:C=sorted(A.possibilities);B.append(f"(Possible options: {', '.join(C)})")
		return '  '.join(B)
class BadOptionUsage(UsageError):
	def __init__(A,option_name,message,ctx=_A):UsageError.__init__(A,message,ctx);A.option_name=option_name
class BadArgumentUsage(UsageError):
	def __init__(A,message,ctx=_A):UsageError.__init__(A,message,ctx)
class FileError(ClickException):
	def __init__(A,filename,hint=_A):
		C=filename;B=hint;D=filename_to_ui(C)
		if B is _A:B='unknown error'
		ClickException.__init__(A,B);A.ui_filename=D;A.filename=C
	def format_message(A):return f"Could not open file {A.ui_filename}: {A.message}"
class Abort(RuntimeError):0
class Exit(RuntimeError):
	__slots__='exit_code',
	def __init__(A,code=0):A.exit_code=code


_H='getint'
_G='getfloat'
_F='getboolean'
_E='list'
_D='float'
_C='int'
_B='bool'
_A=None
from dynaconf.vendor.box.box import Box
class ConfigBox(Box):
	_protected_keys=dir(Box)+[_B,_C,_D,_E,_F,_G,_H]
	def __getattr__(A,item):
		try:return super().__getattr__(item)
		except AttributeError:return super().__getattr__(item.lower())
	def __dir__(A):return super().__dir__()+[_B,_C,_D,_E,_F,_G,_H]
	def bool(C,item,default=_A):
		E=False;B=default;A=item
		try:A=C.__getattr__(A)
		except AttributeError as D:
			if B is not _A:return B
			raise D
		if isinstance(A,(bool,int)):return bool(A)
		if isinstance(A,str)and A.lower()in('n','no','false','f','0'):return E
		return True if A else E
	def int(C,item,default=_A):
		B=default;A=item
		try:A=C.__getattr__(A)
		except AttributeError as D:
			if B is not _A:return B
			raise D
		return int(A)
	def float(C,item,default=_A):
		B=default;A=item
		try:A=C.__getattr__(A)
		except AttributeError as D:
			if B is not _A:return B
			raise D
		return float(A)
	def list(E,item,default=_A,spliter=',',strip=True,mod=_A):
		C=strip;B=default;A=item
		try:A=E.__getattr__(A)
		except AttributeError as F:
			if B is not _A:return B
			raise F
		if C:A=A.lstrip('[').rstrip(']')
		D=[B.strip()if C else B for B in A.split(spliter)]
		if mod:return list(map(mod,D))
		return D
	def getboolean(A,item,default=_A):return A.bool(item,default)
	def getint(A,item,default=_A):return A.int(item,default)
	def getfloat(A,item,default=_A):return A.float(item,default)
	def __repr__(A):return '<ConfigBox: {0}>'.format(str(A.to_dict()))
	def copy(A):return ConfigBox(super().copy())
	def __copy__(A):return ConfigBox(super().copy())


import importlib
import io
import os
import pprint
import sys
import warnings
import webbrowser
from contextlib import suppress
from pathlib import Path

from dynaconf import constants
from dynaconf import default_settings
from dynaconf import LazySettings
from dynaconf import loaders
from dynaconf import settings as legacy_settings
from dynaconf.loaders.py_loader import get_module
from dynaconf.utils import upperfy
from dynaconf.utils.files import read_file
from dynaconf.utils.parse_conf import parse_conf_data
from dynaconf.validator import ValidationError
from dynaconf.validator import Validator
from dynaconf.vendor import click
from dynaconf.vendor import toml


CWD = Path.cwd()
EXTS = ["ini", "toml", "yaml", "json", "py", "env"]
WRITERS = ["ini", "toml", "yaml", "json", "py", "redis", "vault", "env"]

ENC = default_settings.ENCODING_FOR_DYNACONF


def set_settings(ctx, instance=None):
    """Pick correct settings instance and set it to a global variable."""

    global settings

    settings = None

    if instance is not None:
        sys.path.insert(0, ".")
        settings = import_settings(instance)
    elif "FLASK_APP" in os.environ:  # pragma: no cover
        with suppress(ImportError, click.UsageError):
            from flask.cli import ScriptInfo

            flask_app = ScriptInfo().load_app()
            settings = flask_app.config
            click.echo(
                click.style(
                    "Flask app detected", fg="white", bg="bright_black"
                )
            )
    elif "DJANGO_SETTINGS_MODULE" in os.environ:  # pragma: no cover
        sys.path.insert(0, os.path.abspath(os.getcwd()))
        try:
            # Django extension v2
            from django.conf import settings

            settings.DYNACONF.configure()
        except (ImportError, AttributeError):
            # Backwards compatible with old django extension (pre 2.0.0)
            import dynaconf.contrib.django_dynaconf  # noqa
            from django.conf import settings as django_settings

            django_settings.configure()
            settings = django_settings

        if settings is not None:
            click.echo(
                click.style(
                    "Django app detected", fg="white", bg="bright_black"
                )
            )

    if settings is None:

        if instance is None and "--help" not in click.get_os_args():
            if ctx.invoked_subcommand and ctx.invoked_subcommand not in [
                "init",
            ]:
                warnings.warn(
                    "Starting on 3.x the param --instance/-i is now required. "
                    "try passing it `dynaconf -i path.to.settings <cmd>` "
                    "Example `dynaconf -i config.settings list` "
                )
                settings = legacy_settings
            else:
                settings = LazySettings(create_new_settings=True)
        else:
            settings = LazySettings()


def import_settings(dotted_path):
    """Import settings instance from python dotted path.

    Last item in dotted path must be settings instace.

    Example: import_settings('path.to.settings')
    """
    if "." in dotted_path:
        module, name = dotted_path.rsplit(".", 1)
    else:
        raise click.UsageError(
            f"invalid path to settings instance: {dotted_path}"
        )
    try:
        module = importlib.import_module(module)
    except ImportError as e:
        raise click.UsageError(e)
    try:
        return getattr(module, name)
    except AttributeError as e:
        raise click.UsageError(e)


def split_vars(_vars):
    """Splits values like foo=bar=zaz in {'foo': 'bar=zaz'}"""
    return (
        {
            upperfy(k.strip()): parse_conf_data(
                v.strip(), tomlfy=True, box_settings=settings
            )
            for k, _, v in [item.partition("=") for item in _vars]
        }
        if _vars
        else {}
    )


def read_file_in_root_directory(*names, **kwargs):
    """Read a file on root dir."""
    return read_file(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf-8"),
    )


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(read_file_in_root_directory("VERSION"))
    ctx.exit()


def open_docs(ctx, param, value):  # pragma: no cover
    if not value or ctx.resilient_parsing:
        return
    url = "https://dynaconf.com/"
    webbrowser.open(url, new=2)
    click.echo(f"{url} opened in browser")
    ctx.exit()


def show_banner(ctx, param, value):
    """Shows dynaconf awesome banner"""
    if not value or ctx.resilient_parsing:
        return
    set_settings(ctx)
    click.echo(settings.dynaconf_banner)
    click.echo("Learn more at: http://github.com/rochacbruno/dynaconf")
    ctx.exit()


@click.group()
@click.option(
    "--version",
    is_flag=True,
    callback=print_version,
    expose_value=False,
    is_eager=True,
    help="Show dynaconf version",
)
@click.option(
    "--docs",
    is_flag=True,
    callback=open_docs,
    expose_value=False,
    is_eager=True,
    help="Open documentation in browser",
)
@click.option(
    "--banner",
    is_flag=True,
    callback=show_banner,
    expose_value=False,
    is_eager=True,
    help="Show awesome banner",
)
@click.option(
    "--instance",
    "-i",
    default=None,
    envvar="INSTANCE_FOR_DYNACONF",
    help="Custom instance of LazySettings",
)
@click.pass_context
def main(ctx, instance):
    """Dynaconf - Command Line Interface\n
    Documentation: https://dynaconf.com/
    """
    set_settings(ctx, instance)


@main.command()
@click.option(
    "--format", "fileformat", "-f", default="toml", type=click.Choice(EXTS)
)
@click.option(
    "--path", "-p", default=CWD, help="defaults to current directory"
)
@click.option(
    "--env", "-e", default=None, help="Sets the working env in `.env` file"
)
@click.option(
    "--vars",
    "_vars",
    "-v",
    multiple=True,
    default=None,
    help=(
        "extra values to write to settings file "
        "file e.g: `dynaconf init -v NAME=foo -v X=2"
    ),
)
@click.option(
    "--secrets",
    "_secrets",
    "-s",
    multiple=True,
    default=None,
    help=(
        "secret key values to be written in .secrets "
        "e.g: `dynaconf init -s TOKEN=kdslmflds"
    ),
)
@click.option("--wg/--no-wg", default=True)
@click.option("-y", default=False, is_flag=True)
@click.option("--django", default=os.environ.get("DJANGO_SETTINGS_MODULE"))
@click.pass_context
def init(ctx, fileformat, path, env, _vars, _secrets, wg, y, django):
    """Inits a dynaconf project
    By default it creates a settings.toml and a .secrets.toml
    for [default|development|staging|testing|production|global] envs.

    The format of the files can be changed passing
    --format=yaml|json|ini|py.

    This command must run on the project's root folder or you must pass
    --path=/myproject/root/folder.

    The --env/-e is deprecated (kept for compatibility but unused)
    """
    click.echo("⚙️  Configuring your Dynaconf environment")
    click.echo("-" * 42)
    path = Path(path)

    if settings.get("create_new_settings") is True:
        filename = Path("config.py")
        if not filename.exists():
            with open(filename, "w") as new_settings:
                new_settings.write(
                    constants.INSTANCE_TEMPLATE.format(
                        settings_files=[
                            f"settings.{fileformat}",
                            f".secrets.{fileformat}",
                        ]
                    )
                )
            click.echo(
                "🐍 The file `config.py` was generated.\n"
                "  on your code now use `from config import settings`.\n"
                "  (you must have `config` importable in your PYTHONPATH).\n"
            )
        else:
            click.echo(
                f"⁉️  You already have a {filename} so it is not going to be\n"
                "  generated for you, you will need to create your own \n"
                "  settings instance e.g: config.py \n"
                "      from dynaconf import Dynaconf \n"
                "      settings = Dynaconf(**options)\n"
            )
        sys.path.append(str(path))
        set_settings(ctx, "config.settings")

    env = settings.current_env.lower()

    loader = importlib.import_module(f"dynaconf.loaders.{fileformat}_loader")
    # Turn foo=bar=zaz in {'foo': 'bar=zaz'}
    env_data = split_vars(_vars)
    _secrets = split_vars(_secrets)

    # create placeholder data for every env
    settings_data = {}
    secrets_data = {}
    if env_data:
        settings_data[env] = env_data
        settings_data["default"] = {k: "a default value" for k in env_data}
    if _secrets:
        secrets_data[env] = _secrets
        secrets_data["default"] = {k: "a default value" for k in _secrets}

    if str(path).endswith(
        constants.ALL_EXTENSIONS + ("py",)
    ):  # pragma: no cover  # noqa
        settings_path = path
        secrets_path = path.parent / f".secrets.{fileformat}"
        gitignore_path = path.parent / ".gitignore"
    else:
        if fileformat == "env":
            if str(path) in (".env", "./.env"):  # pragma: no cover
                settings_path = path
            elif str(path).endswith("/.env"):  # pragma: no cover
                settings_path = path
            elif str(path).endswith(".env"):  # pragma: no cover
                settings_path = path.parent / ".env"
            else:
                settings_path = path / ".env"
            Path.touch(settings_path)
            secrets_path = None
        else:
            settings_path = path / f"settings.{fileformat}"
            secrets_path = path / f".secrets.{fileformat}"
        gitignore_path = path / ".gitignore"

    if fileformat in ["py", "env"] or env == "main":
        # for Main env, Python and .env formats writes a single env
        settings_data = settings_data.get(env, {})
        secrets_data = secrets_data.get(env, {})

    if not y and settings_path and settings_path.exists():  # pragma: no cover
        click.confirm(
            f"⁉  {settings_path} exists do you want to overwrite it?",
            abort=True,
        )

    if not y and secrets_path and secrets_path.exists():  # pragma: no cover
        click.confirm(
            f"⁉  {secrets_path} exists do you want to overwrite it?",
            abort=True,
        )

    if settings_path:
        loader.write(settings_path, settings_data, merge=True)
        click.echo(
            f"🎛️  {settings_path.name} created to hold your settings.\n"
        )

    if secrets_path:
        loader.write(secrets_path, secrets_data, merge=True)
        click.echo(f"🔑 {secrets_path.name} created to hold your secrets.\n")
        ignore_line = ".secrets.*"
        comment = "\n# Ignore dynaconf secret files\n"
        if not gitignore_path.exists():
            with io.open(str(gitignore_path), "w", encoding=ENC) as f:
                f.writelines([comment, ignore_line, "\n"])
        else:
            existing = (
                ignore_line
                in io.open(str(gitignore_path), encoding=ENC).read()
            )
            if not existing:  # pragma: no cover
                with io.open(str(gitignore_path), "a+", encoding=ENC) as f:
                    f.writelines([comment, ignore_line, "\n"])

        click.echo(
            f"🙈 the {secrets_path.name} is also included in `.gitignore` \n"
            "  beware to not push your secrets to a public repo \n"
            "  or use dynaconf builtin support for Vault Servers.\n"
        )

    if django:  # pragma: no cover
        dj_module, loaded_from = get_module({}, django)
        dj_filename = dj_module.__file__
        if Path(dj_filename).exists():
            click.confirm(
                f"⁉  {dj_filename} is found do you want to add dynaconf?",
                abort=True,
            )
            with open(dj_filename, "a") as dj_file:
                dj_file.write(constants.DJANGO_PATCH)
            click.echo("🎠  Now your Django settings are managed by Dynaconf")
        else:
            click.echo("❌  Django settings file not written.")

    click.echo(
        "🎉 Dynaconf is configured! read more on https://dynaconf.com\n"
        "   Use `dynaconf -i config.settings list` to see your settings\n"
    )


@main.command(name="list")
@click.option(
    "--env", "-e", default=None, help="Filters the env to get the values"
)
@click.option("--key", "-k", default=None, help="Filters a single key")
@click.option(
    "--more",
    "-m",
    default=None,
    help="Pagination more|less style",
    is_flag=True,
)
@click.option(
    "--loader",
    "-l",
    default=None,
    help="a loader identifier to filter e.g: toml|yaml",
)
@click.option(
    "--all",
    "_all",
    "-a",
    default=False,
    is_flag=True,
    help="show dynaconf internal settings?",
)
@click.option(
    "--output",
    "-o",
    type=click.Path(writable=True, dir_okay=False),
    default=None,
    help="Filepath to write the listed values as json",
)
@click.option(
    "--output-flat",
    "flat",
    is_flag=True,
    default=False,
    help="Output file is flat (do not include [env] name)",
)
def _list(env, key, more, loader, _all=False, output=None, flat=False):
    """Lists all user defined config values
    and if `--all` is passed it also shows dynaconf internal variables.
    """
    if env:
        env = env.strip()
    if key:
        key = key.strip()
    if loader:
        loader = loader.strip()

    if env:
        settings.setenv(env)

    cur_env = settings.current_env.lower()

    if cur_env == "main":
        flat = True

    click.echo(
        click.style(
            f"Working in {cur_env} environment ",
            bold=True,
            bg="bright_blue",
            fg="bright_white",
        )
    )

    if not loader:
        data = settings.as_dict(env=env, internal=_all)
    else:
        identifier = f"{loader}_{cur_env}"
        data = settings._loaded_by_loaders.get(identifier, {})
        data = data or settings._loaded_by_loaders.get(loader, {})

    # remove to avoid displaying twice
    data.pop("SETTINGS_MODULE", None)

    def color(_k):
        if _k in dir(default_settings):
            return "blue"
        return "green"

    def format_setting(_k, _v):
        key = click.style(_k, bg=color(_k), fg="white")
        data_type = click.style(
            f"<{type(_v).__name__}>", bg="bright_black", fg="white"
        )
        value = pprint.pformat(_v)
        return f"{key}{data_type} {value}"

    if not key:
        datalines = "\n".join(
            format_setting(k, v)
            for k, v in data.items()
            if k not in data.get("RENAMED_VARS", [])
        )
        (click.echo_via_pager if more else click.echo)(datalines)
        if output:
            loaders.write(output, data, env=not flat and cur_env)
    else:
        key = upperfy(key)
        value = data.get(key)
        if not value:
            click.echo(click.style("Key not found", bg="red", fg="white"))
            return
        click.echo(format_setting(key, value))
        if output:
            loaders.write(output, {key: value}, env=not flat and cur_env)

    if env:
        settings.setenv()


@main.command()
@click.argument("to", required=True, type=click.Choice(WRITERS))
@click.option(
    "--vars",
    "_vars",
    "-v",
    multiple=True,
    default=None,
    help=(
        "key values to be written "
        "e.g: `dynaconf write toml -e NAME=foo -e X=2"
    ),
)
@click.option(
    "--secrets",
    "_secrets",
    "-s",
    multiple=True,
    default=None,
    help=(
        "secret key values to be written in .secrets "
        "e.g: `dynaconf write toml -s TOKEN=kdslmflds -s X=2"
    ),
)
@click.option(
    "--path",
    "-p",
    default=CWD,
    help="defaults to current directory/settings.{ext}",
)
@click.option(
    "--env",
    "-e",
    default="default",
    help=(
        "env to write to defaults to DEVELOPMENT for files "
        "for external sources like Redis and Vault "
        "it will be DYNACONF or the value set in "
        "$ENVVAR_PREFIX_FOR_DYNACONF"
    ),
)
@click.option("-y", default=False, is_flag=True)
def write(to, _vars, _secrets, path, env, y):
    """Writes data to specific source"""
    _vars = split_vars(_vars)
    _secrets = split_vars(_secrets)
    loader = importlib.import_module(f"dynaconf.loaders.{to}_loader")

    if to in EXTS:

        # Lets write to a file
        path = Path(path)

        if str(path).endswith(constants.ALL_EXTENSIONS + ("py",)):
            settings_path = path
            secrets_path = path.parent / f".secrets.{to}"
        else:
            if to == "env":
                if str(path) in (".env", "./.env"):  # pragma: no cover
                    settings_path = path
                elif str(path).endswith("/.env"):
                    settings_path = path
                elif str(path).endswith(".env"):
                    settings_path = path.parent / ".env"
                else:
                    settings_path = path / ".env"
                Path.touch(settings_path)
                secrets_path = None
                _vars.update(_secrets)
            else:
                settings_path = path / f"settings.{to}"
                secrets_path = path / f".secrets.{to}"

        if (
            _vars and not y and settings_path and settings_path.exists()
        ):  # pragma: no cover  # noqa
            click.confirm(
                f"{settings_path} exists do you want to overwrite it?",
                abort=True,
            )

        if (
            _secrets and not y and secrets_path and secrets_path.exists()
        ):  # pragma: no cover  # noqa
            click.confirm(
                f"{secrets_path} exists do you want to overwrite it?",
                abort=True,
            )

        if to not in ["py", "env"]:
            if _vars:
                _vars = {env: _vars}
            if _secrets:
                _secrets = {env: _secrets}

        if _vars and settings_path:
            loader.write(settings_path, _vars, merge=True)
            click.echo(f"Data successful written to {settings_path}")

        if _secrets and secrets_path:
            loader.write(secrets_path, _secrets, merge=True)
            click.echo(f"Data successful written to {secrets_path}")

    else:  # pragma: no cover
        # lets write to external source
        with settings.using_env(env):
            # make sure we're in the correct environment
            loader.write(settings, _vars, **_secrets)
        click.echo(f"Data successful written to {to}")


@main.command()
@click.option(
    "--path", "-p", default=CWD, help="defaults to current directory"
)
def validate(path):  # pragma: no cover
    """Validates Dynaconf settings based on rules defined in
    dynaconf_validators.toml"""
    # reads the 'dynaconf_validators.toml' from path
    # for each section register the validator for specific env
    # call validate

    path = Path(path)

    if not str(path).endswith(".toml"):
        path = path / "dynaconf_validators.toml"

    if not path.exists():  # pragma: no cover  # noqa
        click.echo(click.style(f"{path} not found", fg="white", bg="red"))
        sys.exit(1)

    validation_data = toml.load(open(str(path)))

    success = True
    for env, name_data in validation_data.items():
        for name, data in name_data.items():
            if not isinstance(data, dict):  # pragma: no cover
                click.echo(
                    click.style(
                        f"Invalid rule for parameter '{name}'",
                        fg="white",
                        bg="yellow",
                    )
                )
            else:
                data.setdefault("env", env)
                click.echo(
                    click.style(
                        f"Validating '{name}' with '{data}'",
                        fg="white",
                        bg="blue",
                    )
                )
                try:
                    Validator(name, **data).validate(settings)
                except ValidationError as e:
                    click.echo(
                        click.style(f"Error: {e}", fg="white", bg="red")
                    )
                    success = False

    if success:
        click.echo(click.style("Validation success!", fg="white", bg="green"))
    else:
        click.echo(click.style("Validation error!", fg="white", bg="red"))
        sys.exit(1)


if __name__ == "__main__":  # pragma: no cover
    main()
